@@ -69,6 +69,11 @@ class EC2VM(VMResource):
         self.vm_password = None
         self.ssh_conn = None
         self.volume_id = None
+        # load instance spec while cannot retrive it automatically
+        self.cpus_cfg = params.get('cpus')
+        self.memory_cfg = params.get('memory')
+        self.disks_cfg = params.get('disks')
+        self.net_bandwidth_cfg = params.get('net_bandwidth')
         self.is_created = False
         self.another_ip = None
         self.run_uuid = params.get('run_uuid')
@@ -108,6 +113,9 @@ class EC2VM(VMResource):
             LOG.info('Cannot determin root device name, use default {}'.format(self.root_device_name))
 
         volume_size = 10
+        if self.hibernation_support:
+            # extend disk size to 20 in case no space to create swap
+            volume_size = 20
         vm_kwargs = {
             'BlockDeviceMappings':[
                 {
@@ -150,6 +158,9 @@ class EC2VM(VMResource):
             'HibernationOptions':{
                 'Configured': self.hibernation_support
             },
+            'MetadataOptions':{
+                'InstanceMetadataTags': 'enabled'
+            },
             "UserData":self.user_data
         }
         if self.efa_support:
@@ -248,6 +259,9 @@ class EC2VM(VMResource):
 
     @property
     def disk_count(self):
+        if self.disks_cfg:
+            LOG.info("Setting in cfg {}".format(self.disks_cfg))
+            return self.disks_cfg
         default_disks = 1
         instance_types_dict = self.client.describe_instance_types(InstanceTypes=[self.instance_type])
         instance_types = instance_types_dict.get('InstanceTypes')
@@ -262,6 +276,9 @@ class EC2VM(VMResource):
 
     @property
     def net_bandwidth(self):
+        if self.net_bandwidth_cfg:
+            LOG.info("Setting in cfg {}".format(self.net_bandwidth_cfg))
+            return self.net_bandwidth_cfg
         default_bandwidth = 5
         instance_types_dict = self.client.describe_instance_types(InstanceTypes=[self.instance_type])
         instance_types = instance_types_dict.get('InstanceTypes')
@@ -405,15 +422,21 @@ class EC2VM(VMResource):
                 InstanceId=self.ec2_instance.id, DryRun=False)
             return True
         except ClientError as err:
-            LOG.error("Failed to send_diagnostic_interrupt to %s" %
-                      self.ec2_instance.id)
-            return False
+            LOG.error("Failed to send_diagnostic_interrupt to {}:{}".format(self.ec2_instance.id, err))
+            if 'UnsupportedOperation' not in str(err):
+                return False
+        raise UnSupportedAction('This instance does not support send nmi')
 
     def send_hibernation(self):
         if not self.hibernation_support:
             LOG.info("this instance not support hibernation")
             return False
-        return self.stop(hibernate=True)
+        # wait system is ready for hiberation
+        for i in range(10):
+            time.sleep(20)
+            if self.stop(hibernate=True):
+                return True
+        return False
 
     def get_console_log(self, silient=False):
         ret = None
