@@ -42,7 +42,8 @@ def get_service(api_name, api_version, scopes, key_file_location):
 
 def verify_token(token: str, audience: str) -> dict:
     request = google.auth.transport.requests.Request()
-    payload = id_token.verify_token(token, request=request, audience=audience, clock_skew_in_seconds=150)
+    payload = id_token.verify_token(
+        token, request=request, audience=audience, clock_skew_in_seconds=150)
     return payload
 
 
@@ -139,6 +140,16 @@ class GCPVM(VMResource):
         f_ip = self.data['networkInterfaces'][0]['accessConfigs'][0]['natIP']
         return f_ip
 
+    @property
+    def is_secure_boot(self):
+        if self.arch == "x86_64":
+            return True
+        return False
+
+    @property
+    def is_uefi_boot(self):
+        return True
+
     def create(self, sev=False, wait=False):
         # Get image.
         source_disk_image = get_image(self.service_v1, self.project,
@@ -167,6 +178,10 @@ class GCPVM(VMResource):
                 }
             }],
 
+            "displayDevice": {
+                "enableDisplay": True
+            },
+
             # Specify a network interface with NAT to access the public
             # internet.
             'networkInterfaces': [{
@@ -192,6 +207,10 @@ class GCPVM(VMResource):
                 'provisioningModel': 'SPOT',
                 'instanceTerminationAction': 'DELETE'
             },
+            'shieldedInstanceConfig': {
+                'enableIntegrityMonitoring': True,
+                'enableVtpm': True
+            },
         }
 
         if sev:
@@ -201,6 +220,9 @@ class GCPVM(VMResource):
             config['disks'][0]['interface'] = 'NVME'
             config['networkInterfaces'][0]['nicType'] = 'GVNIC'
 
+        if self.is_secure_boot:
+            config['shieldedInstanceConfig']['enableSecureBoot'] = True
+
         operation = self.service_v1.instances().insert(project=self.project,
                                                        zone=self.zone,
                                                        body=config).execute()
@@ -296,6 +318,9 @@ class GCPVM(VMResource):
         self.data = self.vm_name
         return self.data['status']
 
+    def get_state(self):
+        return self._get_status()
+
     def is_started(self):
         return self._get_status() == 'RUNNING'
 
@@ -327,9 +352,6 @@ class GCPVM(VMResource):
     def disk_count(self):
         raise NotImplementedError
 
-    def get_state(self):
-        raise NotImplementedError
-
     def send_nmi(self):
         raise UnSupportedAction('No such operation in gcp')
 
