@@ -157,7 +157,7 @@ class PrismApi(PrismSession):
         endpoint = urljoin(self.base_url, "vms")
         if vm_name == None:
             vm_name = self.vm_name
-	# Attach image.
+  # Attach image.
         images = self.list_images()
         vmdisk_uuid = ""
         for image in images['entities']:
@@ -191,6 +191,7 @@ runcmd:
         if self.user_data != None:
             user_data = self.user_data
             user_data += user_data_ssh_key
+            self.user_data = None
         logging.info('userdata for creating vm is \n %s' % user_data)
         # Attach user script.
         user_script=[]
@@ -211,7 +212,7 @@ runcmd:
                 'boot_device_type': 'disk',
                 'uefi_boot': self.if_uefi_boot,
                 'secure_boot': self.if_secure_boot
-	        },
+          },
             'memory_mb':
             self.memory * 1024,
             'name':
@@ -258,6 +259,9 @@ runcmd:
             'vm_nics': network_uuids,
             'machine_type': self.machine_type
         }
+        logging.info("==============vm data================>")
+        logging.info(data)
+        logging.info("<==============vm data================")
         return self.make_request(endpoint, 'post', data=data)
 
     def create_vm_ISO_kickstart(self, single_nic=True, vm_name=None):
@@ -265,7 +269,7 @@ runcmd:
             vm_name = self.vm_name + vm_name
         logging.debug("Create VM by ISO kickstart")
         endpoint = urljoin(self.base_url, "vms")
-	# Attach image.
+  # Attach image.
         images = self.list_images()
         vmdisk_uuid = []
         for image in images['entities']:
@@ -340,7 +344,7 @@ runcmd:
         
     def delete_vm(self, vm_uuid):
         logging.debug("Delete VM")
-        endpoint = urljoin(self.base_url, "vms/%s" % vm_uuid)
+        endpoint = urljoin(self.base_url, "vms/%s/?delete_snapshots=true" % vm_uuid)
         return self.make_request(endpoint, 'delete')
 
     def restart_vm(self, vm_uuid):
@@ -515,7 +519,7 @@ runcmd:
                 if is_cdrom:
                     images = self.list_images()
                     for image in images['entities']:
-                        if image['name'] == 'seed_iso':
+                        if image['name'] == 'seed.iso':
                             vmdisk_uuid = image['vm_disk_id']
                 else:
                     logging.info('OPPS.Clone disk form img_service has not yet support.')
@@ -529,7 +533,15 @@ runcmd:
                     'storage_container_uuid': self.storage_container_uuid
                     }}]}
             else:
-                logging.info('OPPS.Clone disk form adsf file path has not yet support.')
+                data = {"vm_disks": [{"disk_address":{"device_bus":device_bus, "device_index":device_index},
+                    "is_cdrom": "true",
+                    "is_empty": "false",
+                    'vm_disk_clone': {
+                    'disk_address': {
+                        'ndfs_filepath': 'seed.iso'
+                    },
+                    'storage_container_uuid': self.storage_container_uuid
+                    }}]}
         logging.info('data to attach disk is \n'+str(data))
         return self.make_request(endpoint, 'post', data=data)
 
@@ -662,6 +674,8 @@ class NutanixVM(VMResource):
         self.host_username = params['Credential']['host_username']
         self.host_password = params['Credential']['host_password']
         self.net_bandwidth = params['VM']['net_bandwidth']
+        self.machine_type = params['VM']['machine_type']
+        self.if_secure_boot = params['VM']['if_secure_boot']
         self.user_data = None
         self.vm1_ip = ''
 
@@ -823,12 +837,15 @@ class NutanixVM(VMResource):
         vm = self.get_vm_by_filter('vm_name', vm_name)
         return vm
 
-    def delete(self, wait=True):
-        logging.info("Delete VM")
-        res = self.prism.delete_vm(self.data.get('uuid'))
+    def delete(self, wait=True, uuid='default'):
+        logging.info("Delete VM for %s" % uuid)
+        if uuid == 'default':
+            res = self.prism.delete_vm(self.data.get('uuid'))
+        else:
+            res = self.prism.delete_vm(uuid)
         if wait:
             self.wait_for_status(
-                res['task_uuid'], 30,
+                res['task_uuid'], 60,
                 "Timed out waiting for server to get deleted.")
 
     def start(self, wait=True):
@@ -850,9 +867,9 @@ class NutanixVM(VMResource):
         if wait:
             self.wait_for_status(
                 res['task_uuid'], 60,
-                "Timed out waiting for server to get stopped.")
+                "Timed out waiting for task status of stopping vm.")
             for count in utils_lib.iterate_timeout(
-                    30, "Timed out waiting for server to get stopped."):
+                    120, "Timed out waiting for server to get stopped."): #Timed out in CI log, change to 120
                 if self.is_stopped():
                     break
 
@@ -876,7 +893,7 @@ class NutanixVM(VMResource):
             res = self.prism.migrate_vm(self.data.get('uuid'))
         if wait:
             self.wait_for_status(
-                res['taskUuid'], 120,
+                res['taskUuid'], 600,
                 "Timed out waiting for VM to complete migration.")
         self._data = None
 
@@ -892,6 +909,9 @@ class NutanixVM(VMResource):
         self._data = None
         return self.data.get('power_state')
 
+    def get_state(self):
+        return self._get_status()
+
     def is_started(self):
         return self._get_status() == 'on'
 
@@ -954,7 +974,7 @@ class NutanixVM(VMResource):
                 self.stop(wait=True)
             res = self.prism.update_vcpu(self.data.get('uuid'), vcpu_num_target)
         self.wait_for_status(
-            res['task_uuid'], 60,
+            res['task_uuid'], 120,
             "Timed out waiting for VM to complete vCPU number updating.")
         if self.is_stopped():
             self.start(wait=True)
@@ -976,7 +996,7 @@ class NutanixVM(VMResource):
             self.stop(wait=True)
         res = self.prism.update_core(self.data.get('uuid'), core_num_target)
         self.wait_for_status(
-            res['task_uuid'], 60,
+            res['task_uuid'], 120,
             "Timed out waiting for VM to complete core number per vCPU updating.")
         self.start(wait=True)
         for count in utils_lib.iterate_timeout(
@@ -1000,12 +1020,12 @@ class NutanixVM(VMResource):
                 self.stop(wait=True)
             res = self.prism.update_memory(self.data.get('uuid'), mem_gb_target)
         self.wait_for_status(
-            res['task_uuid'], 60,
+            res['task_uuid'], 120,
             "Timed out waiting for VM to complete memory capacity (GB) updating.")
         if self.is_stopped():
             self.start(wait=True)
         for count in utils_lib.iterate_timeout(
-                60, "Timed out waiting for verify memory capacity (GB) updating."):
+                120, "Timed out waiting for verify memory capacity (GB) updating."):
             if self.exists() and self.get_memory_size() == mem_gb_target:
                 break
 
@@ -1041,7 +1061,7 @@ class NutanixVM(VMResource):
             logging.info("VM cpu passthrough has changed successfully.")
             self.start(wait=True)
             for count in utils_lib.iterate_timeout(
-                    60, "Timed out waiting for verify cpu passthrough changing."):
+                    120, "Timed out waiting for verify cpu passthrough changing."):
                 if enabled:
                     if self.get_cpu_passthrough(enabled=True):
                         break
@@ -1092,7 +1112,7 @@ class NutanixVM(VMResource):
             logging.info("vGPU has assigned to VM successfully.")
             self.start(wait=True)
             for count in utils_lib.iterate_timeout(
-                    10, "Timed out waiting for verify vGPU assignment."):
+                    60, "Timed out waiting for verify vGPU assignment."):
                 if self.get_vgpu_info():
                     break
 
@@ -1113,7 +1133,7 @@ class NutanixVM(VMResource):
             logging.info("vGPU has deassigned from VM successfully.")
             self.start(wait=True)
             for count in utils_lib.iterate_timeout(
-                    10, "Timed out waiting for verify vGPU deassignment."):
+                    60, "Timed out waiting for verify vGPU deassignment."):
                 if not self.get_vgpu_info():
                     break
 
@@ -1126,18 +1146,18 @@ class NutanixVM(VMResource):
 
         if wait:
             self.wait_for_status(
-                res['task_uuid'], 30,
+                res['task_uuid'], 120,  #Timed out in CI log, change to 120
                 "Timed out attaching disk.")
 
     def detach_disk(self, device_bus, vmdisk_uuid, device_index, wait=True):
         '''
-        Attach disk according to vmdisk_uuid
+        Detach disk according to vmdisk_uuid
         '''
         res = self.prism.detach_disk(self.data.get('uuid'), device_bus, vmdisk_uuid, device_index)
         if wait:
             self.wait_for_status(
-                res['task_uuid'], 30,
-                "Timed out attaching disk.")
+                res['task_uuid'], 180,  #Timed out in CI log, change to 180
+                "Timed out detaching disk.")
     
     def attach_block(self, disk, target, wait=True, timeout=120):
         raise NotImplementedError
@@ -1158,7 +1178,7 @@ class NutanixVM(VMResource):
         res = self.prism.detach_nic(self.data.get('uuid'), nic_mac)
         if wait:
             self.wait_for_status(
-                res['task_uuid'], 60,
+                res['task_uuid'], 120,      ##"Timed out waiting for detaching NIC" in CI log, cannot reproduce, change to 120.
                 "Timed out waiting for detaching NIC.")
 
     def get_nic(self):
@@ -1168,9 +1188,6 @@ class NutanixVM(VMResource):
     def get_console_log(self):
         raise UnSupportedAction('No such operation in nutanix')
     
-    def get_state(self):
-        raise NotImplementedError
-
     def is_exist(self):
         raise NotImplementedError
 
@@ -1273,6 +1290,7 @@ class NutanixVolume(StorageResource):
 
     def get_state(self):
         raise NotImplementedError
+
     def is_exist(self):
         if self.vm.show():
             logging.info("Instance disk: {}".format(self.vm.show()['vm_disk_info']))
@@ -1286,4 +1304,4 @@ class NutanixVolume(StorageResource):
         res = self.prism.expand_disk(disk_uuid=disk_uuid, disk_size=origin_disk_size+expand_size, device_index=disk_index)
         self.vm.wait_for_status(
                 res['task_uuid'], 60,
-                "Timed out waiting for restoring VM.")
\ No newline at end of file
+                "Timed out waiting for restoring VM.")
