@@ -37,8 +37,8 @@ def init_args():
                     help='skip cases, add --strict for skipping exactly', required=False)
     parser.add_argument('--verifydoc', dest='verifydoc', action='store_true',
                     help='verify or show case doc only', required=False)
-    parser.add_argument('--uploaddoc', dest='uploaddoc', default=None, action='store',
-                    help='upload doc to specific test case management system, eg. polarion', required=False)
+    parser.add_argument('--dumpdoc', dest='dumpdoc', default=None, action='store',
+                    help='dump doc to yaml file', required=False)
     parser.add_argument('--filter_by', dest='filter_by', default='case_name', action='store',
                     help="filter by 'case_name'(default),'case_tag','case_file','component','bugzilla_id',\
                         'is_customer_case','testplan','maintainer','description','key_steps',\
@@ -59,15 +59,21 @@ def init_args():
                     help='specify azure to run azure image check only', required=False)
     parser.add_argument('--platform_profile', dest='platform_profile', default=None, action='store',
                     help='specify platform profile if enable os-tests provison vms self, only supports aws for now', required=False)
+    parser.add_argument('--no-cleanup', dest='no_cleanup', action='store_true',
+                    help='debug purpose, skip cleanup phase at exit, do not use it in normal test', required=False)
+    parser.add_argument('--proxy_url', dest='proxy_url', default=None, action='store',
+                    help='specify it if pkg/repo url is internal only, format IP:PORT', required=False)
     args = parser.parse_args()
     return args
 
 def init_provider(params=None):
+    # this init provider from cfg
     vms = []
     disks = []
     nics = []
     supported_platforms = ['aws', 'openstack', 'ali', 'nutanix', 'google', 'libvirt', 'openshift']
     provider = params['Cloud']['provider']
+    os.environ['INFRA_PROVIDER'] = provider
     if not provider:
         LOG.info("no provider found in profile".format(provider, supported_platforms))
         sys.exit(1)
@@ -103,6 +109,24 @@ def init_provider(params=None):
 
     return vms, disks, nics
 
+def init_provider_from_guest(test_instance):
+    # this init provider from system itself
+    if os.getenv('INFRA_PROVIDER'):
+        test_instance.log.info("already set provider to {}".format(os.getenv('INFRA_PROVIDER')))
+        return True
+    provider = 'UNKNOW'
+    if is_aws(test_instance):
+        provider = 'aws'
+    if is_openstack(test_instance):
+        provider = 'openstack'
+    if is_ali(test_instance):
+        provider = 'ali'
+    if is_ahv(test_instance):
+        provider = 'nutanix'
+    if is_gcp(test_instance):
+        provider = 'google'
+    os.environ['INFRA_PROVIDER'] = provider
+
 def init_ssh(params=None, timeout=600, interval=10, log=None, rmt_node=None):
     if log is None:
         LOG_FORMAT = '%(levelname)s:%(message)s'
@@ -114,6 +138,7 @@ def init_ssh(params=None, timeout=600, interval=10, log=None, rmt_node=None):
     ssh.rmt_user = params['remote_user']
     ssh.rmt_password = params['remote_password']
     ssh.rmt_keyfile = params['remote_keyfile']
+    ssh.rmt_proxy = params['proxy_url']
     ssh.log = log
     ssh.timeout = timeout
     ssh.interval = interval
@@ -153,10 +178,15 @@ def init_connection(test_instance, timeout=600, interval=10, rmt_node=None, vm=N
             if ssh.rmt_node == rmt_node:
                 ssh_exists = True
                 ssh_num = i
-                ret, _, _ = ssh.cli_run(cmd='uname -r')
-                if ret == 0:
-                    test_instance.log.info("connection is live")
-                    is_active = True
+                if hasattr(ssh, 'is_active'):
+                    # this is avaiable in tipset >= 0.2.0
+                    is_active = ssh.is_active()
+                else:
+                    #can remove below when all use ssh.is_active()
+                    ret, _, _ = ssh.cli_run(cmd='uname -r')
+                    if ret == 0:
+                        test_instance.log.info("connection is live")
+                        is_active = True
                 break
         for tmp_ssh in test_instance.SSHs:
             if tmp_ssh.rmt_node == test_instance.params['remote_node']:
@@ -290,6 +320,10 @@ def init_case(test_instance):
             test_instance.ssh_timeout = 1200
         if not test_instance.vm.exists():
             test_instance.vm.create()
+        if hasattr(test_instance.vm, 'get_state') and 'stopping' in str(test_instance.vm.get_state()):
+            for count in iterate_timeout(
+                600, "Timed out waiting for getting server stopped."):
+                if test_instance.vm.is_stopped(): break
         if test_instance.vm.is_stopped():
             test_instance.vm.start(wait=True)
         test_instance.params['remote_port'] = test_instance.vm.port or 22
@@ -319,6 +353,8 @@ def init_case(test_instance):
         test_instance.node_info = node_info_data
     else:
         _, test_instance.node_info = get_cfg(cfg_file=node_info)
+    init_provider_from_guest(test_instance)
+    core_file_check(test_instance)
 
 def finish_case(test_instance):
     """finish case
@@ -327,30 +363,6 @@ def finish_case(test_instance):
     """
     pass
 
-def case_to_polarion(case=None,cfg=None):
-    from os_tests.libs.polarion import PolarionCase
-    polarion_case = PolarionCase(cfg=cfg)
-    LOG.info("upload {}".format(case.id()))
-    yaml_data = {}
-    try:
-        yaml_data = load(case._testMethodDoc, Loader=Loader)
-        if not hasattr(yaml_data,'get'):
-            yaml_data = {}
-            yaml_data['case_name'] = case.id()
-        else:
-            yaml_data['case_name'] = case.id()
-    except Exception as err:
-        yaml_fail = err
-        yaml_data['case_name'] = case.id()
-        LOG.info("please verify its docstring before uploading")
-        return False
-    polarion_case.casedoc = yaml_data
-    if polarion_case.query_case():
-        polarion_case.update_case()
-    else:
-        polarion_case.add_new()
-    return True
-
 def filter_case_doc(case=None, patterns=None, skip_patterns=None, filter_field='case_name', strict=False, verify_doc=False ):
     if patterns is None and skip_patterns is None and not verify_doc:
         return True
@@ -416,10 +428,7 @@ def filter_case_doc(case=None, patterns=None, skip_patterns=None, filter_field='
                     is_skip = True
                     break
     if verify_doc and is_select and not is_skip:
-        expect_fields = ['case_tags', 'title', 'importance', 'subsystem_team', 'automation_drop_down', 
-                        'linked_work_items', 'automation_field', 'setup_teardown', 'environment', 'case_name', 
-                        'component', 'bug_id', 'is_customer_case', 'testplan', 'maintainer', 'description', 
-                        'key_steps', 'expected_result', 'debug_want']
+        from tipset.libs import polarion_adm
         print("="*20)
         print(case.id())
         print(case._testMethodDoc)
@@ -427,9 +436,7 @@ def filter_case_doc(case=None, patterns=None, skip_patterns=None, filter_field='
         if  yaml_fail:
             print(yaml_fail)
             return is_select and not is_skip
-        for i in expect_fields:
-            if yaml_data.get(i) == None:
-                print('missing {}'.format(i))
+        polarion_adm.verify_doc(casedoc = yaml_data)
     return is_select and not is_skip
 
 def msg_to_syslog(test_instance, cmd='sudo virt-what', msg=None):
@@ -445,9 +452,9 @@ def msg_to_syslog(test_instance, cmd='sudo virt-what', msg=None):
     output = ''
     if cmd:
         output = run_cmd(test_instance, cmd)
-    cmd = 'sudo echo os-tests:"{} {}" | systemd-cat -p info'.format(msg, output)
+    cmd = 'sudo echo os-tests:"{} \n{}" | systemd-cat -p info'.format(msg, output)
     run_cmd(test_instance, cmd, expect_ret=0)
-    cmd = "sudo bash -c 'echo \"{} {}\" > /dev/kmsg'".format(msg,output)
+    cmd = "sudo bash -c 'echo \"{} \n{}\" > /dev/kmsg'".format(msg, output)
     run_cmd(test_instance, cmd, expect_ret=0)
 
 def run_cmd(test_instance,
@@ -509,6 +516,8 @@ def run_cmd(test_instance,
     status = None
     output = None
     exception_hit = False
+    run_err = ''
+    ssh_index = 0
 
     try:
         if test_instance.is_rmt:
@@ -523,6 +532,7 @@ def run_cmd(test_instance,
                 ssh.log = test_instance.log
                 if ssh.rmt_node == rmt_node:
                     SSH = ssh
+                    ssh_index = i
                     break
             test_instance.log.info("CMD: {} on {}".format(cmd, rmt_node))
             status, output = SSH.remote_excute(cmd, timeout, redirect_stdout=rmt_redirect_stdout, redirect_stderr=rmt_redirect_stderr,rmt_get_pty=rmt_get_pty)
@@ -540,12 +550,17 @@ def run_cmd(test_instance,
         test_instance.log.error("Run cmd failed: {}".format(err))
         status = None
         exception_hit = True
+        run_err = str(err)
 
     if exception_hit:
         test_cmd = 'uname -a'
         test_instance.log.info("Test system is alive via cmd:{}. If still fail, check no hang or panic happens.".format(test_cmd))
         try:
             if test_instance.is_rmt:
+                if 'Key-exchange timed out' in run_err:
+                    test_instance.log.info('reconnect to remote because it acheived certain number of packets or bytes sent or received using this session')
+                    test_instance.SSHs[ssh_index].create_connection()
+                    SSH = test_instance.SSHs[ssh_index]
                 status, output = SSH.remote_excute(test_cmd, timeout)
                 status, output = SSH.remote_excute(cmd, timeout, redirect_stdout=rmt_redirect_stdout, redirect_stderr=rmt_redirect_stderr,rmt_get_pty=rmt_get_pty)
             else:
@@ -590,27 +605,21 @@ def run_cmd(test_instance,
             (status, expect_not_ret))
     if expect_kw is not None:
         for key_word in expect_kw.split(','):
-            if output.count('\n') > 5:
-                find_list = re.findall('.*{}.*'.format(key_word), output)
-            else:
-                find_list = re.findall('.*{}.*'.format(key_word), output)
-            if len(find_list) > 0:
+            find_list = re.findall('.*{}.*'.format(key_word), output)
+            if find_list:
                 test_instance.log.info('expected "{}" found in "{}"'.format(key_word, ''.join(find_list)))
             else:
-                if output.count('\n') > 5:
+                if output is not None and output.count('\n') > 5:
                     test_instance.fail('expected "{}" not found in output(check debug log as too many lines)'.format(key_word))
                 else:
                     test_instance.fail('expected "{}" not found in "{}"'.format(key_word,output))
     if expect_not_kw is not None:
         for key_word in expect_not_kw.split(','):
-            if output.count('\n') > 5:
-                find_list = re.findall('\n.*{}.*\n'.format(key_word), output)
-            else:
-                find_list = re.findall('.*{}.*'.format(key_word), output)
-            if len(find_list) == 0:
+            find_list = re.findall('.*{}.*'.format(key_word), output)
+            if not find_list:
                 test_instance.log.info('Unexpected "{}" not found in output'.format(key_word))
             else:
-                if output.count('\n') > 5:
+                if output is not None and output.count('\n') > 5:
                     test_instance.fail('Unexpected "{}" found in {}'.format(key_word, ''.join(find_list)))
                 else:
                     test_instance.fail('Unexpected "{}" found in "{}"'.format(key_word,output))
@@ -715,6 +724,8 @@ def is_sev_enabled(test_instance):
         enabled: return True
         N/A: return False
     '''
+    if not test_instance.vm:
+        test_instance.skipTest("Skip as no VM provisioned.")
     if test_instance.vm.provider == 'google':
         audience = 'aud'
         cmd = "curl -s -H 'Metadata-Flavor: Google' \
@@ -759,8 +770,11 @@ def is_aws(test_instance, action=None):
         aws: return True
         other: return False
     '''
-    output = run_cmd(test_instance, "sudo cat /sys/devices/virtual/dmi/id/bios_*", expect_ret=0)
-    if 'amazon' in output.lower():
+    if is_pkg_installed(test_instance, pkg_name='virt-what', cancel_case=False, is_install=False):
+        output = run_cmd(test_instance, "sudo bash -c 'cat /sys/devices/virtual/dmi/id/bios_*; virt-what'", expect_ret=0)
+    else:
+        output = run_cmd(test_instance, "sudo cat /sys/devices/virtual/dmi/id/bios_*", expect_ret=0)
+    if any(x in output.lower() for x in ['aws', 'amazon']):
         test_instance.log.info("AWS system.")
         return True
     else:
@@ -801,7 +815,10 @@ def is_ali(test_instance, action=None):
         aws: return True
         other: return False
     '''
-    output = run_cmd(test_instance, "sudo cat /sys/devices/virtual/dmi/id/product_*", expect_ret=0)
+    if is_pkg_installed(test_instance, pkg_name='virt-what', cancel_case=False, is_install=False):
+        output = run_cmd(test_instance, "sudo bash -c 'cat /sys/devices/virtual/dmi/id/bios_*; virt-what'", expect_ret=0)
+    else:
+        output = run_cmd(test_instance, "sudo cat /sys/devices/virtual/dmi/id/product_*", expect_ret=0)
     if 'alibaba' in output.lower():
         test_instance.log.info("Ali system.")
         return True
@@ -840,7 +857,10 @@ def is_ahv(test_instance, action=None):
         ahv: return True
         other: return False
     '''
-    output = run_cmd(test_instance, "sudo cat /sys/devices/virtual/dmi/id/product_*", expect_ret=0)
+    if is_pkg_installed(test_instance, pkg_name='virt-what', cancel_case=False, is_install=False):
+        output = run_cmd(test_instance, "sudo virt-what", expect_ret=0)
+    else:
+        output = run_cmd(test_instance, "sudo cat /sys/devices/virtual/dmi/id/product_*", expect_ret=0)
     if 'ahv' in output.lower():
         test_instance.log.info("Nutanix AHV system.")
         return True
@@ -860,7 +880,10 @@ def is_gcp(test_instance, action=None):
         aws: return True
         other: return False
     '''
-    output = run_cmd(test_instance, "sudo cat /sys/devices/virtual/dmi/id/bios_*", expect_ret=0)
+    if is_pkg_installed(test_instance, pkg_name='virt-what', cancel_case=False, is_install=False):
+        output = run_cmd(test_instance, "sudo virt-what", expect_ret=0)
+    else:
+        output = run_cmd(test_instance, "sudo cat /sys/devices/virtual/dmi/id/bios_*", expect_ret=0)
     if 'google' in output.lower():
         test_instance.log.info("gcp system.")
         return True
@@ -941,7 +964,7 @@ def is_cmd_exist(test_instance, cmd=None, is_install=True, cancel_case=False):
         pkg_name = pkg_names[0].split(':')[-1].strip(' ')
     else:
         test_instance.skipTest("Unable to retrive {} owned by which pkg".format(cmd))
-    run_cmd(test_instance, "sudo yum install -y {}".format(pkg_name), expect_ret=0, timeout=180)
+    run_cmd(test_instance, "sudo yum install -y {}".format(pkg_name), expect_ret=0, timeout=720)
     return True
 
 def is_pkg_installed(test_instance, pkg_name=None, is_install=True, cancel_case=False, timeout=120):
@@ -1073,12 +1096,15 @@ def check_log(test_instance, log_keyword, log_cmd="journalctl -b 0", match_word_
         test_instance.log.info("Loading baseline data file from {}".format(baseline_file))
         baseline_dict = json.load(fh)
     run_cmd(test_instance, '\n')
-    check_cmd = log_cmd
 
+    check_cmd = log_cmd + '|grep -Ev "{}"'.format(test_instance.id())
+    if 'test_check' in test_instance.id():
+        check_cmd = check_cmd + '|grep -Ev test_check'
     if match_word_exact:
-        check_cmd = check_cmd + '|grep -iw %s' % log_keyword
+        check_cmd = check_cmd + '|grep -iw "{}"'.format(log_keyword)
     if skip_words:
-        check_cmd = check_cmd + '|grep -Ev "{}"'.format(skip_words.replace(',','|'))
+        check_cmd = check_cmd + '|grep -Ev "{}"'.format(skip_words.replace(',', '|'))
+
     ret = False
     if cursor is not None:
         out = run_cmd(test_instance,
@@ -1169,91 +1195,53 @@ def find_word(test_instance, check_str, log_keyword, baseline_dict=None, skip_wo
     if len(tmp_list) == 0:
         test_instance.log.info("No {} found after skipped {}!".format(log_keyword, skip_words))
         return True, []
-    unknow_log = deepcopy(tmp_list)
-    # compare 2 string, if similary over fail_rate, consider it as same.
-    fail_rate = 70
+    unknow_log = list(set(deepcopy(tmp_list)))
+
     no_fail = True
-    check_done = False
-    for line1 in tmp_list:
-        # this round go through with regex
-        find_it = False
+    for line1 in set(tmp_list):
+        test_instance.log.info("Checking:{}".format(line1))
+        found_it = False
         if baseline_dict:
             for basekey in baseline_dict:
+                # this round go through with regex
                 for sub_basekey_content in baseline_dict[basekey]["content"].split(';'):
-                    check_done = False
                     if sub_basekey_content and re.search(sub_basekey_content, line1):
-                        if baseline_dict[basekey]["status"] == 'active':
-                            test_instance.log.info("Found a similar log matched in baseline.")
-                            find_it = True
-                        else:
-                            test_instance.log.info("Found a similar log matched in baseline. But it is not active, please check manually")
-                            find_it = False
-                            no_fail = False
-                            check_done = True
-                            break
-                        trigger = baseline_dict[basekey]["trigger"]
-                        if trigger and re.search(trigger,check_str,flags=re.I):
-                            test_instance.log.info("Guess it is expected because trigger keywords found '{}'".format(trigger))
-                            find_it = True
-                        elif trigger:
-                            test_instance.log.info("Guess it is unexpected because trigger keywords not found '{}'".format(trigger))
-                            find_it = False
-                        test_instance.log.info("log:{}, base:{}".format(line1, sub_basekey_content))
-                        test_instance.log.info("ID:%s Baseline analyze:%s Branch:%s Status:%s Link:%s Path:%s" %
-                             (basekey,
-                              baseline_dict[basekey]["analyze"],
-                              baseline_dict[basekey]["branch"],
-                              baseline_dict[basekey]["status"],
-                              baseline_dict[basekey]["link"],
-                              baseline_dict[basekey]["path"]))
-                        check_done = True
+                        found_it = True
+                        matched_msg = basekey
+                        test_instance.log.info("regex found in baseline:{}".format(basekey))
                         break
-                if find_it:
+            if not found_it:
+                # this round compare the content
+                # compare 2 strings, if similary over pass_rate, consider it as same.
+                pass_rate = 70
+                for basekey in baseline_dict:
+                    line1_tmp = line1
+                    line2_tmp = baseline_dict[basekey]["content"]
+                    line1_tmp, line2_tmp = clean_sentence(test_instance, line1_tmp, line2_tmp)
+                    seq = difflib.SequenceMatcher(
+                        None, a=line1_tmp, b=line2_tmp)
+                    same_rate = seq.ratio() * 100
+                    if same_rate > pass_rate:
+                        test_instance.log.info("content similar rate:{} over {} baseline:{}".format(same_rate,pass_rate,basekey))
+                        matched_msg = basekey
+                        found_it = True
+                        break
+            if found_it:
+                basekey = matched_msg
+                if baseline_dict[basekey]["status"] != 'active':
+                    test_instance.log.info("Found a similar log {} matched in baseline. But it is not active, please check manually".format(basekey))
+                    no_fail = False
+                trigger = baseline_dict[basekey]["trigger"]
+                if trigger and re.search(trigger,check_str,flags=re.I):
+                    test_instance.log.info("Guess it is expected because trigger keywords found '{}'".format(trigger))
                     unknow_log.remove(line1)
-                    break
-                if check_done:
-                    break
-    tmp_list = deepcopy(unknow_log)
-    for line1 in tmp_list:
-        # this round go through with content compare
-        find_it = False
-        if baseline_dict:
-            for basekey in baseline_dict:
-                line1_tmp = line1
-                line2_tmp = baseline_dict[basekey]["content"]
-                line1_tmp, line2_tmp = clean_sentence(test_instance, line1_tmp, line2_tmp)
-                seq = difflib.SequenceMatcher(
-                    None, a=line1_tmp, b=line2_tmp)
-                same_rate = seq.ratio() * 100
-                if same_rate > fail_rate:
-                    test_instance.log.info(
-                        "Compare result rate: %d same, maybe it is not a new one", same_rate)
-                    test_instance.log.info("Guest: %s Baseline: %s", line1,
-                             baseline_dict[basekey]["content"])
-                    test_instance.log.info("ID:%s Baseline analyze:%s Branch:%s Status:%s Link:%s Path:%s" %
-                             (basekey,
-                              baseline_dict[basekey]["analyze"],
-                              baseline_dict[basekey]["branch"],
-                              baseline_dict[basekey]["status"],
-                              baseline_dict[basekey]["link"],
-                              baseline_dict[basekey]["path"]))
-                    trigger = baseline_dict[basekey]["trigger"]
-                    if trigger and re.search(trigger,check_str,flags=re.I):
-                        test_instance.log.info("Guess it is expected because trigger keywords found '{}'".format(trigger))
-                        find_it = True
-                    elif trigger:
-                        test_instance.log.info("Guess it is unexpected because trigger keywords not found '{}'".format(trigger))
-                        find_it = False
-                        break
-                    if baseline_dict[basekey]["status"] == 'active':
-                        find_it = True
-                    else:
-                        test_instance.log.info("Find a similar issue which should be already fixed, please check manually.")
-                        find_it = False
-                        no_fail = False
-                    break
-            if find_it:
-                unknow_log.remove(line1)
+                elif trigger:
+                    test_instance.log.info("Guess it is unexpected because trigger keywords not found '{}'".format(trigger))
+                    no_fail = False
+                elif not trigger and no_fail:
+                    unknow_log.remove(line1)
+                test_instance.log.info(baseline_dict[basekey])
+                continue
     if unknow_log:
         test_instance.log.info("Below items are unknow!\n{}".format(unknow_log))
         no_fail = False
@@ -1449,4 +1437,63 @@ def check_attribute(target, attributes, test_instance=None, cancel_case=True):
             else:
                 func_write(msg)
             return False
-    return True
\ No newline at end of file
+    return True
+
+def core_file_check(test_instance=None):
+    '''
+    when there is core file exists, collect it to test result dir for further debugging
+    '''
+    cmd = 'sudo ls /var/lib/systemd/coredump/core*'
+    core_files = run_cmd(test_instance, cmd, msg='check if core file exists')
+    if 'No such file or directory' not in core_files:
+        test_instance.log.info('Please attached core files when report bugs')
+        for core_file in core_files.split('\n'):
+            core_file = core_file.strip('\n')
+            if not core_file:
+                continue
+            cmd = 'sudo chmod 766 {}'.format(core_file)
+            run_cmd(test_instance, cmd, expect_ret=0)
+            if test_instance.params.get('remote_node') is not None:
+                test_instance.log.info('retrive {} from remote to {}'.format(core_file, test_instance.log_dir))
+                test_instance.SSH.get_file(rmt_file=core_file,local_file='{}/attachments/{}'.format(test_instance.log_dir,os.path.basename(core_file)))
+            else:
+                cmd = "cp {} {}/attachments/{}".format(core_file, test_instance.log_dir,os.path.basename(core_file) )
+                run_cmd(test_instance, cmd, msg='save {} to {}'.format(core_file, test_instance.log_dir))
+        run_cmd(test_instance, 'sudo rm -rf /var/lib/systemd/coredump/core*', msg='clean up core files')
+        cmd = 'sudo journalctl -b0'
+        run_cmd(test_instance, cmd, msg='get traceback from journal')
+
+def get_test_disk(test_instance=None):
+        '''
+        Look for non-boot disk to do test
+        '''
+        test_disk = None
+        cmd = "lsblk -r --output NAME,MOUNTPOINT|awk -F' ' '{if($2) printf\"%s \",$1}'"
+        output = run_cmd(test_instance, cmd, expect_ret=0)
+        mount_disks = output.split(' ')
+        cmd = 'lsblk -d --output NAME|grep -v NAME'
+        output = run_cmd(test_instance, cmd, expect_ret=0)
+        disk_list = output.split('\n')
+        for disk in disk_list:
+            disk_in_use = False
+            if not disk:
+                continue
+            for mount_disk in mount_disks:
+                if disk in mount_disk:
+                    test_instance.log.info('Disk is mounted: {}'.format(disk))
+                    disk_in_use = True
+                    break
+            if not disk_in_use:
+                cmd = 'sudo wipefs -a /dev/{}'.format(disk) #comment this line for bz2074486
+                ret = run_cmd(test_instance, cmd, ret_status=True, timeout=1800,msg='test can clean fs on {}'.format(disk))
+                if ret == 0:
+                    test_disk = disk
+                    break
+                else:
+                    test_instance.log.info('Cannot clean fs on {} - skip'.format(disk))
+                    continue
+        if test_disk:
+            test_instance.log.info('Test disk is found: {}'.format(test_disk))
+        else:
+             test_instance.skipTest("No free disk for testing.")
+        return test_disk
\ No newline at end of file
