@@ -8,6 +8,7 @@ import time
 class TestGeneralCheck(unittest.TestCase):
     def setUp(self):
         utils_lib.init_case(self)
+        utils_lib.msg_to_syslog(self)
         if self.id().endswith(('test_check_rpm_V_differences', 'test_check_rpm_V_missing')):
             rpm_V_file = '/tmp/{}_rpm_V.log'.format(self.run_uuid)
             self.output = utils_lib.run_cmd(self, 'cat {}'.format(rpm_V_file), msg="check if output exists")
@@ -24,10 +25,10 @@ class TestGeneralCheck(unittest.TestCase):
                 cmd = 'sudo rpm -V {} > {} 2>&1'.format(' '.join(check_list), rpm_V_file)
                 utils_lib.run_cmd(self, cmd, msg="verify pkgs", timeout=600)
                 self.output = utils_lib.run_cmd(self, 'cat {}'.format(rpm_V_file), expect_ret=0, msg="check if output exists again")
-        if self.id().endswith(('test_check_systemd_analyze_verify_missing', 'test_check_systemd_analyze_verify_deprecated_unsafe', 'test_check_systemd_analyze_verify_obsolete', 'test_check_systemd_analyze_verify_ordering_cycle', 'test_check_journalctl_service_unknown_lvalue')):
+        if any(x in self.id().lower() for x in ['systemd_analyze_verify', 'journalctl_service_unknown']):    
             check_file = self.utils_dir + '/systemd_analyze_services.sh'
             check_file_tmp = '/tmp/systemd_analyze_services.sh'
-            if self.params['remote_node'] is not None:
+            if self.params.get('remote_node') is not None:
                 cmd = 'ls -l {}'.format(check_file_tmp)
                 ret = utils_lib.run_cmd(self, cmd, ret_status=True, msg='check if {} exists'.format(check_file))
                 if ret != 0:
@@ -142,6 +143,8 @@ class TestGeneralCheck(unittest.TestCase):
             the clocksource matched to the system is expected.
             Xen - xen,tsc,hpet,acpi_pm,
             aarch64 - arch_sys_counter,
+            ppc64le - timebase,
+            s390x - tod,
             Microsoft - hyperv_clocksource_tsc_page,acpi_pm,
             AuthenticAMD - kvm-clock,tsc,acpi_pm,
             GenuineIntel - kvm-clock,tsc,acpi_pm
@@ -153,6 +156,10 @@ class TestGeneralCheck(unittest.TestCase):
             expect_clocks = 'xen,tsc,hpet,acpi_pm'
         elif 'aarch64' in output:
             expect_clocks = 'arch_sys_counter'
+        elif 'ppc64le' in output and 'KVM' in output and not utils_lib.is_metal(self):
+            expect_clocks = 'timebase'
+        elif 's390x' in output and 'KVM' in output and not utils_lib.is_metal(self):
+            expect_clocks = 'tod'
         elif 'Microsoft' in output:
             expect_clocks = 'hyperv_clocksource_tsc_page,acpi_pm'
         elif 'AuthenticAMD' in output and 'KVM' in output and not utils_lib.is_metal(self):
@@ -192,6 +199,8 @@ available_clocksource'
         debug_want:
             journal log
         """
+        if utils_lib.is_ahv(self):
+            self.skipTest("Skip test as already covered in test_nutanix_vm.test_check_firstlaunch_time and test_check_reboot_time")
         max_boot_time = self.params.get('max_boot_time')
         boot_time_sec = utils_lib.getboottime(self)
         utils_lib.compare_nums(self, num1=boot_time_sec, num2=max_boot_time, ratio=0, msg="Compare with cfg specified max_boot_time")
@@ -401,7 +410,7 @@ available_clocksource'
             No segmentation fault found.
         '''
         utils_lib.is_cmd_exist(self, cmd='dmidecode')
-        if self.params['remote_node'] is not None:
+        if self.params.get('remote_node') is not None:
             binfile = '/tmp/dmidecode_debug.bin'
             cmd = "sudo dmidecode --dump-bin {}".format(binfile)
             utils_lib.run_cmd(self, cmd, msg='save dmidecode_debug.bin for debug purpose, please attach it if file bug')
@@ -651,7 +660,7 @@ itlb_multihit|grep -v 'no microcode'|grep -v retbleed|sed 's/:/^/' | column -t -
         debug_want:
             journalctl log
         """
-        utils_lib.check_log(self, 'Traceback,Backtrace', skip_words='test_check_journal_calltrace', rmt_redirect_stdout=True)
+        utils_lib.check_log(self, 'Traceback,Backtrace', rmt_redirect_stdout=True)
 
     def test_check_journalctl_cannot(self):
         '''
@@ -662,9 +671,9 @@ itlb_multihit|grep -v 'no microcode'|grep -v retbleed|sed 's/:/^/' | column -t -
         component:
             journal
         bugzilla_id:
-            
-        customer_case_id:
-            
+            1975897,2026544
+        is_customer_case:
+            True 
         polarion_id:
             n/a
         maintainer:
@@ -676,7 +685,7 @@ itlb_multihit|grep -v 'no microcode'|grep -v retbleed|sed 's/:/^/' | column -t -
         expected_result:
             No new unknown 'Could not' log found.
         '''
-        utils_lib.check_log(self, 'Could not,can not', skip_words='test_check_journalctl_cannot', rmt_redirect_stdout=True)
+        utils_lib.check_log(self, 'Could not,can not', rmt_redirect_stdout=True)
 
     def test_check_journalctl_conflict(self):
         '''
@@ -701,7 +710,7 @@ itlb_multihit|grep -v 'no microcode'|grep -v retbleed|sed 's/:/^/' | column -t -
         expected_result:
             No conflict log found.
         '''
-        utils_lib.check_log(self, 'conflict', skip_words='test_check_journalctl_conflict', rmt_redirect_stdout=True)
+        utils_lib.check_log(self, 'conflict', rmt_redirect_stdout=True)
 
     def test_check_journalctl_denied(self):
         '''
@@ -726,7 +735,7 @@ itlb_multihit|grep -v 'no microcode'|grep -v retbleed|sed 's/:/^/' | column -t -
         expected_result:
             No new unknown denied log found.
         '''
-        utils_lib.check_log(self, 'denied', skip_words='test_check_journalctl_denied', rmt_redirect_stdout=True)
+        utils_lib.check_log(self, 'denied', rmt_redirect_stdout=True)
 
     def test_check_journalctl_disabled(self):
         """
@@ -754,7 +763,7 @@ itlb_multihit|grep -v 'no microcode'|grep -v retbleed|sed 's/:/^/' | column -t -
             # journalctl
         """
         expect_not_kws = 'KASLR disabled'
-        utils_lib.check_log(self, expect_not_kws, skip_words='test_check_journalctl_disabled', rmt_redirect_stdout=True)
+        utils_lib.check_log(self, expect_not_kws, rmt_redirect_stdout=True)
 
     def test_check_journalctl_dumpedcore(self):
         """
@@ -778,23 +787,7 @@ itlb_multihit|grep -v 'no microcode'|grep -v retbleed|sed 's/:/^/' | column -t -
         debug_want:
             attach core files under '/var/lib/systemd/coredump/' if report bz
         """
-        cmd = 'sudo ls  /var/lib/systemd/coredump/core*'
-        core_files = utils_lib.run_cmd(self, cmd, msg='check if core file generated')
-        if 'No such file or directory' not in core_files:
-            self.log.info('Please attached core files when repor bugs')
-            for core_file in core_files.split('\n'):
-                core_file = core_file.strip('\n')
-                if not core_file:
-                    continue
-                cmd = 'sudo chmod 766 {}'.format(core_file)
-                utils_lib.run_cmd(self, cmd, expect_ret=0)
-                if self.params['remote_node'] is not None:
-                    self.log.info('retrive {} from remote to {}'.format(core_file, self.log_dir))
-                    self.SSH.get_file(rmt_file=core_file,local_file='{}/attachments/{}'.format(self.log_dir,os.path.basename(core_file)))
-                else:
-                    cmd = "cp {} {}/attachments/{}".format(core_file, self.log_dir,os.path.basename(core_file) )
-                    utils_lib.run_cmd(self, cmd, msg='save {} to {}'.format(core_file, self.log_dir))
-        utils_lib.check_log(self, 'dumped core', skip_words='test_check_journalctl_dumpedcore', rmt_redirect_stdout=True)
+        utils_lib.check_log(self, 'dumped core', rmt_redirect_stdout=True)
 
     def test_check_journalctl_error(self):
         """
@@ -820,7 +813,7 @@ itlb_multihit|grep -v 'no microcode'|grep -v retbleed|sed 's/:/^/' | column -t -
         debug_want:
             dmesg
         """
-        utils_lib.check_log(self, 'error', skip_words='test_check,UpdateGSErrors', rmt_redirect_stdout=True)
+        utils_lib.check_log(self, 'error', skip_words='UpdateGSErrors', rmt_redirect_stdout=True)
 
     def test_check_journalctl_fail(self):
         '''
@@ -843,7 +836,7 @@ itlb_multihit|grep -v 'no microcode'|grep -v retbleed|sed 's/:/^/' | column -t -
         expected_result:
             No new unknown fail log found.
         '''
-        utils_lib.check_log(self, 'fail', skip_words='test_check,nofail', rmt_redirect_stdout=True)
+        utils_lib.check_log(self, 'fail', skip_words='nofail', rmt_redirect_stdout=True)
 
     def test_check_journalctl_not_found(self):
         '''
@@ -869,7 +862,7 @@ itlb_multihit|grep -v 'no microcode'|grep -v retbleed|sed 's/:/^/' | column -t -
         expected_result:
             No new unknown not file found log found.
         '''
-        utils_lib.check_log(self, 'not found,no such', skip_words='test_check_journalctl_not_found', rmt_redirect_stdout=True)
+        utils_lib.check_log(self, 'not found,no such', rmt_redirect_stdout=True)
 
     def test_check_journalctl_unexpected(self):
         '''
@@ -894,7 +887,7 @@ itlb_multihit|grep -v 'no microcode'|grep -v retbleed|sed 's/:/^/' | column -t -
         expected_result:
             No new unknown unexpected log found.
         '''
-        utils_lib.check_log(self, 'unexpected', skip_words='test_check_journalctl_unexpected', rmt_redirect_stdout=True)
+        utils_lib.check_log(self, 'unexpected', rmt_redirect_stdout=True)
 
     def test_check_journalctl_warn(self):
         """
@@ -920,7 +913,7 @@ itlb_multihit|grep -v 'no microcode'|grep -v retbleed|sed 's/:/^/' | column -t -
         debug_want:
             dmesg
         """
-        utils_lib.check_log(self, 'warn', skip_words='test_check',rmt_redirect_stdout=True)
+        utils_lib.check_log(self, 'warn', rmt_redirect_stdout=True)
 
     def test_check_journalctl_invalid(self):
         """
@@ -947,7 +940,7 @@ itlb_multihit|grep -v 'no microcode'|grep -v retbleed|sed 's/:/^/' | column -t -
         polarion_id:
         bz:1750417
         '''
-        utils_lib.check_log(self, 'invalid', skip_words="Invalid user,invalid user,test_check", rmt_redirect_stdout=True)
+        utils_lib.check_log(self, 'invalid', skip_words="Invalid user,invalid user", rmt_redirect_stdout=True)
 
     def test_check_journalctl_service_unknown_lvalue(self):
         """
@@ -975,7 +968,8 @@ itlb_multihit|grep -v 'no microcode'|grep -v retbleed|sed 's/:/^/' | column -t -
             - service unit file content
             - output from 'systemd-analyze verify $service'
         """
-        utils_lib.run_cmd(self, 'cat {}'.format(self.systemd_analyze_verify_file),expect_ret=0, expect_not_kw='Unknown lvalue', msg='Check there is no "Unknown lvalue" keyword')
+        cmd = 'cat {}'.format(self.systemd_analyze_verify_file)
+        utils_lib.check_log(self,'Unknown lvalue',log_cmd=cmd,rmt_redirect_stdout=True)
 
     def test_check_locale(self):
         '''
@@ -1047,6 +1041,9 @@ itlb_multihit|grep -v 'no microcode'|grep -v retbleed|sed 's/:/^/' | column -t -
                     "size" : 4286578688, <-- 4GiB is correct
                         "size" : 4286578688,
         '''
+        if utils_lib.is_ahv(self):
+            self.skipTest("Skip test as already covered in test_nutanix_vm.test_check_memory_size")
+
         utils_lib.is_cmd_exist(self, cmd='lshw')
         base_memory = utils_lib.get_memsize(self)
         cmd = 'sudo lshw -json'
@@ -1120,13 +1117,14 @@ itlb_multihit|grep -v 'no microcode'|grep -v retbleed|sed 's/:/^/' | column -t -
         '''
         case_name:
             test_check_lspci_nvme
-
         case_priority:
             1
+        automation_field:
+            https://github.com/virt-s1/os-tests/blob/master/os_tests/tests/test_general_check.py
         component:
             kernel
-        bugzilla_id:
-            1656862
+        bug_id:
+            bugzilla_1656862,bugzilla_2173504
         polarion_id:
             n/a
         maintainer:
@@ -1138,8 +1136,14 @@ itlb_multihit|grep -v 'no microcode'|grep -v retbleed|sed 's/:/^/' | column -t -
             2. # lsblk -d|grep nvme|wc -l
         expected_result:
             The nums are equal.
+        debug_want: |
+            If the test fail, please attch the test log which includes below commands output.
+            cmds: dmesg, sudo lspci -vvv, lsblk
         '''
         utils_lib.is_cmd_exist(self, cmd='lspci')
+        cmds = ["dmesg", "sudo lspci -vvv", "lsblk"]
+        for cmd in cmds:
+            utils_lib.run_cmd(self, cmd, msg='retrive debug info')
         lspci_cmd = "lspci|grep 'Non-Volatile memory'|wc -l"
         lsblk_cmd = "lsblk -d|grep nvme|wc -l"
         lspci_out = utils_lib.run_cmd(self, lspci_cmd,cancel_not_kw='0', msg="Check nvme pci device")
@@ -1681,6 +1685,59 @@ in cmdline as bug1859088")
         utils_lib.run_cmd(self, cmd, expect_not_kw='ordering cycle', msg='Check there is no ordering cycle in journal log')
         utils_lib.run_cmd(self, 'cat {}'.format(self.systemd_analyze_verify_file),expect_ret=0, expect_not_kw='ordering cycle', msg='Check there is no ordering cycle which may block boot up')
 
+    def test_check_systemd_analyze_verify_instead(self):
+        """
+        case_name:
+            test_check_systemd_analyze_verify_instead
+        case_tags:
+            systemd
+        case_status:
+            Approved
+        title:
+            check service has no option which will be removed soon in unit file 
+        importance:
+            low
+        subsystem_team:
+            sst_virtualization_cloud
+        automation_drop_down:
+            Automated
+        linked_work_items:
+            N/A
+        automation_field:
+            https://github.com/virt-s1/os-tests/blob/master/os_tests/tests/test_general_check.py
+        setup_teardown:
+            N/A
+        environment:
+            N/A
+        component:
+            systemd
+        bug_id:
+            bugzilla_2155468
+        is_customer_case:
+            False
+        testplan:
+            N/A
+        test_type:
+            Functional
+        test_level:
+            Component
+        maintainer:
+            xiliang@redhat.com
+        description: |
+            check service unit file has no option which will be removed soon or has "legacy directory" and permission denied settings.
+        key_steps: |
+            # systemd-analyze verify $service
+        expected_result: |
+            - no instead option found
+            - no "legacy directory" used
+            - no "Permission denied" found
+        debug_want: |
+            - output from 'systemd-analyze verify $service'
+            - output from 'systemctl cat $failservice'
+        """
+        cmd = 'cat {}'.format(self.systemd_analyze_verify_file)
+        utils_lib.check_log(self,'instead,legacy directory,Permission denied',log_cmd=cmd,rmt_redirect_stdout=True)
+
     def test_check_tsc_deadline_timer(self):
         """
         case_name:
@@ -1975,11 +2032,13 @@ current_device"
         utils_lib.run_cmd(self,
                     'sudo insights-client --status',
                     cancel_kw="System is registered",
-                    msg="Please register system or add user to '/etc/insights-client/insights-client.conf'")
-        utils_lib.run_cmd(self,
-                    'sudo insights-client --check-result',
-                    expect_ret=0,
-                    msg="checking system")
+                    cancel_not_kw="machine is NOT registered",
+                    msg="Please register system or add subscription info to '/etc/insights-client/insights-client.conf'")
+        out = utils_lib.run_cmd(self, 'sudo insights-client --check-result', msg="checking system")
+        if "multiple hosts detected" in out:
+            cmd = "sudo bash -c 'insights-client --unregister && insights-client --register'"
+            utils_lib.run_cmd(self, cmd, msg="try to unregister and register system", timeout=240)
+        utils_lib.run_cmd(self, 'sudo insights-client --check-result', expect_ret=0, msg="checking system")
         result_out = utils_lib.run_cmd(self,
                     'sudo insights-client --show-result',
                     expect_ret=0,
@@ -2028,13 +2087,13 @@ current_device"
         utils_lib.run_cmd(self,cmd,msg="clean up old sos report")
         utils_lib.is_cmd_exist(self, cmd='sosreport')
         cmd = "sudo sosreport --batch"
-        utils_lib.run_cmd(self,cmd,expect_ret=0,msg="test sosreport",timeout=400)
+        utils_lib.run_cmd(self,cmd,expect_ret=0,msg="test sosreport",timeout=900)
         cmd = 'sudo ls /var/tmp/sos*.xz'
         sosfile = utils_lib.run_cmd(self, cmd, expect_ret=0)
         sosfile = sosfile.strip('\n')
         cmd = 'sudo chmod 766 {}'.format(sosfile)
         utils_lib.run_cmd(self, cmd, expect_ret=0)
-        if self.params['remote_node'] is not None:
+        if self.params.get('remote_node') is not None:
             self.log.info('retrive {} from remote to {}'.format(sosfile, self.log_dir))
             self.SSH.get_file(rmt_file=sosfile,local_file='{}/attachments/{}'.format(self.log_dir,os.path.basename(sosfile)))
         else:
@@ -2068,9 +2127,16 @@ current_device"
         """
         try:
             if utils_lib.is_sev_enabled(self):
-                utils_lib.run_cmd(self, 'dmesg', expect_ret=0,
-                                  expect_kw='AMD Memory Encryption Features active: SEV',
-                                  msg="Check there is 'AMD Memory Encryption Features active: SEV' in dmesg before run 'perf top'")
+                # https://gitlab.com/redhat/centos-stream/src/kernel/centos-stream-9/-/merge_requests/875/diffs?commit_id=ea66ccfe756058c054f6c32b30f79e69e2b77c08#1314bf7c9c25b9572d0a973f6be52499f0478e85
+                v = float(utils_lib.get_product_id(self))
+                if 8.8 <= v < 9.0 or v >=9.2:
+                    utils_lib.run_cmd(self, 'dmesg', expect_ret=0,
+                                      expect_kw='Memory Encryption Features active: AMD SEV',
+                                      msg="Check there is 'Memory Encryption Features active: AMD SEV' in dmesg before run 'perf top'")
+                else:
+                    utils_lib.run_cmd(self, 'dmesg', expect_ret=0,
+                                      expect_kw='AMD Memory Encryption Features active: SEV',
+                                      msg="Check there is 'AMD Memory Encryption Features active: SEV' in dmesg before run 'perf top'")
             else:
                 self.skipTest('SEV is not enabled')
         except NotImplementedError:
