@@ -1,8 +1,10 @@
+from multiprocessing import cpu_count
 import unittest
 import re
 from os_tests.libs import utils_lib
 import time
 import os
+from datetime import datetime
 
 class TestGeneralTest(unittest.TestCase):
     def setUp(self):
@@ -145,6 +147,71 @@ current_clocksource'
         cmd = "sudo cpupower frequency-info"
         utils_lib.run_cmd(self, cmd, expect_ret=0, expect_not_kw='core dumped')
 
+    def test_cpu_hotplug_no_workload(self):
+        """
+        case_name:
+            test_cpu_hotplug_no_workload
+        case_tags:
+            kernel
+        case_status:
+            Approved
+        title:
+            simple case to offline, online cpu without workload
+        importance:
+            low
+        subsystem_team:
+            sst_virtualization_cloud
+        automation_drop_down:
+            Automated
+        linked_work_items:
+            N/A
+        automation_field:
+            https://github.com/virt-s1/os-tests/blob/master/os_tests/tests/test_general_test.py
+        setup_teardown:
+            N/A
+        environment:
+            N/A
+        component:
+            kernel
+        bug_id:
+            bugzilla_2160354,bugzilla_1716271,kcs_4463681
+        is_customer_case:
+            True
+        testplan:
+            N/A
+        test_type:
+            Functional
+        test_level:
+            Component
+        maintainer:
+            xiliang@redhat.com
+        description: |
+            simple case to offline, online cpu without workload
+        key_steps: |
+            # echo 0 > /sys/devices/system/cpu/cpu1/online
+            # lscpu|grep Off-line
+            # echo 1 > /sys/devices/system/cpu/cpu1/online
+        expected_result: |
+            - can offline, online cpu1 without any error, no panic, no call trace
+        debug_want: |
+            - dmesg
+        """
+        cmd = "grep processor /proc/cpuinfo | wc -l"
+        cpu_counts = int(utils_lib.run_cmd(self, cmd, expect_ret=0,
+                                           msg = "Get cpu counts"))
+        if int(cpu_counts) < 2:
+            self.skipTest("only run when cpu count >= 2")
+        utils_lib.run_cmd(self, 'lscpu', expect_ret=0)
+        cmd = "sudo bash -c 'echo 0 > /sys/devices/system/cpu/cpu1/online'"
+        utils_lib.run_cmd(self, cmd, expect_ret=0)
+        cmd = 'lscpu'
+        utils_lib.run_cmd(self, cmd, expect_ret=0, expect_kw='Off-line')
+        cmd = "sudo bash -c 'echo 1 > /sys/devices/system/cpu/cpu1/online'"
+        utils_lib.run_cmd(self, cmd, expect_ret=0)
+        cmd = 'lscpu'
+        utils_lib.run_cmd(self, cmd, expect_ret=0, expect_not_kw='Off-line')
+        utils_lib.run_cmd(self, 'dmesg', expect_ret=0)
+
     def test_dracut_f_v(self):
         '''
         case_name:
@@ -291,8 +358,8 @@ int main(int argc, char *argv[])
         key_steps:
             1. # wget https://github.com/redis/redis/files/5717040/redis_8124.c.txt
             2. # mv redis_8124.c.txt redis_8124.c
-            3. # gcc -o reproduce redis_8124.c
-            4. # systemd-run --scope -p MemoryLimit=550M ./reproduce
+            3. # gcc -o os_tests_redis redis_8124.c
+            4. # systemd-run --scope -p MemoryLimit=550M ./os_tests_redis
         expected_result:
             Your kernel looks fine.
         '''
@@ -303,14 +370,14 @@ int main(int argc, char *argv[])
         utils_lib.is_cmd_exist(self, cmd='wget', cancel_case=True)
         redis_src = self.utils_dir + '/redis_8124.c'
         redis_src_tmp = '/tmp/redis_8124.c'
-        if self.params['remote_node'] is not None:
+        if self.params.get('remote_node') is not None:
             self.log.info('Copy {} to remote'.format(redis_src))
             self.SSH.put_file(local_file=redis_src, rmt_file=redis_src_tmp)
         else:
             cmd = "sudo cp -f {} {}".format(redis_src, redis_src_tmp)
             utils_lib.run_cmd(self, cmd, expect_ret=0, timeout=120)
-        cmd_list = ['gcc -o /tmp/reproduce /tmp/redis_8124.c',
-                    'sudo systemd-run --scope -p MemoryLimit=550M /tmp/reproduce']
+        cmd_list = ['gcc -o /tmp/os_tests_redis /tmp/redis_8124.c',
+                    'sudo systemd-run --scope -p MemoryLimit=550M /tmp/os_tests_redis']
         for cmd in cmd_list:
             out = utils_lib.run_cmd(self, cmd, expect_ret=0, timeout=120)
         if 'Your kernel looks fine' not in out:
@@ -449,6 +516,12 @@ int main(int argc, char *argv[])
         des: "subscription-manager config" output should equal "subscription-manager config --list"
         '''
         utils_lib.is_cmd_exist(self, 'subscription-manager')
+        utils_lib.run_cmd(self, "sudo subscription-manager config",
+                          expect_ret=0,
+                          msg="Run 'subscription-manager config' first to skip the yum/dnf plugins warning, \
+the warning is only in qcow2 image as the plugins have been disabled by default, more info please refer to: \
+https://bugzilla.redhat.com/show_bug.cgi?id=2107502")
+        
         cmd1 = "sudo subscription-manager config > /tmp/sm_config.log"
         out1 = utils_lib.run_cmd(self, cmd1, expect_ret=0, msg='get {} output'.format(cmd1))
         cmd2 = "sudo subscription-manager config --list > /tmp/sm_config_list.log"
@@ -487,10 +560,10 @@ int main(int argc, char *argv[])
                int x;
                free(&x);
                }
-            3. # gcc -g -o pp test.c
+            3. # gcc -g -o os_tests_coretest test.c
             4. # ./pp
         expect_result:
-            pp crashed and new core file is generated under /var/lib/systemd/coredump
+            os_tests_coretest crashed and new core file is generated under /var/lib/systemd/coredump
         debug_want:
             - journal log
         """
@@ -514,10 +587,10 @@ int main(int argc, char *argv[])
         utils_lib.is_cmd_exist(self, 'gcc')
         cmd = "echo '{}' > {}".format(test_str, test_file)
         utils_lib.run_cmd(self, cmd, expect_ret=0, msg='generate {}'.format(test_file))
-        cmd = "gcc -g -o /tmp/pp {}".format(test_file)
+        cmd = "gcc -g -o /tmp/os_tests_coretest {}".format(test_file)
         utils_lib.run_cmd(self, cmd, expect_ret=0)
-        utils_lib.run_cmd(self, 'ulimit -c unlimited;/tmp/pp', msg='run it to trigger core dump')
-        utils_lib.run_cmd(self, 'sudo ls /var/lib/systemd/coredump/core.pp*', expect_ret=0, msg='check core file generated')
+        utils_lib.run_cmd(self, 'ulimit -c unlimited;/tmp/os_tests_coretest', msg='run it to trigger core dump')
+        utils_lib.run_cmd(self, 'sudo ls /var/lib/systemd/coredump/core.os_tests_core*', expect_ret=0, msg='check core file generated')
         utils_lib.check_log(self, "warn,fail", log_cmd='journalctl -b0', cursor=self.cursor, rmt_redirect_stdout=True)
 
     def test_podman_build_image(self):
@@ -1058,8 +1131,238 @@ if __name__ == "__main__":
         utils_lib.run_cmd(self, f'sudo nitro-cli terminate-enclave --enclave-id {EnclaveID}',
                           expect_kw='"Terminated": true', msg='terminate enclave')
 
+    def test_able_to_sync_chrony_server(self):
+        """
+        case_name:
+            test_able_to_sync_chrony_server
+        case_tags:
+            time keeping
+        case_status:
+            approved
+        title:
+            Check guest able to sync time to chrony server
+        importance:
+            medium
+        subsystem_team:
+            sst_virtualization_cloud
+        automation_drop_down:
+            Automated
+        linked_work_items:
+            N/A
+        automation_field:
+            https://github.com/virt-s1/os-tests/blob/master/os_tests/tests/test_general_test.py
+        setup_teardown:
+            Recover chrony default configure in tear down.
+        environment:
+            N/A
+        component:
+            time keeping
+        bug_id:
+            N/A
+        is_customer_case:
+            False
+        testplan:
+            N/A
+        test_type:
+            Functional
+        test_level:
+            Component
+        maintainer:
+            minl@redhat.com
+        description: |
+            Check guest able to sync time to chrony server
+        key_steps: |
+            # Change guest time through date command
+            # edit config file: /etc/chrony.conf, then restart chrony service
+                - Add "server clock.redhat.com"
+                - Comment "makestep 1.0 3" line
+                - Add "makestep 1 -1"
+                - Restart chronyd service
+            #Check guest time sync to chrony server
+        expected_result: |
+            - guest time can sync to chrony server.
+        debug_want: |
+            - N/A
+        """
+        utils_lib.run_cmd(self, "timedatectl status")
+        utils_lib.run_cmd(self, "sudo hwclock --verbose")
+        #Check if VM has chronyc command
+        utils_lib.is_cmd_exist(self, 'chronyc')
+        #Change VM time
+        utils_lib.run_cmd(self, 'sudo date -s 00:00:00', expect_ret=0)
+        utils_lib.run_cmd(self, 'date "+%H:%M:%S"', expect_kw="00:00:")
+        #Check if chrony source available, if not, add available source.
+        chrony_source = utils_lib.run_cmd(self, "chronyc sources -v", expect_ret=0)
+        utils_lib.run_cmd(self, 'sudo cp /etc/chrony.conf /etc/chrony.conf.backup')
+        if not re.search('\^\*', chrony_source):
+            cmd1 = 'sudo sed -i "/Please consider joining the pool/a\server clock.redhat.com" \
+                /etc/chrony.conf'
+            utils_lib.run_cmd(self, cmd1, expect_ret=0)
+        cmd2 = 'sudo sed -i "/makestep 1.0 3/c\makestep 1 -1" /etc/chrony.conf'
+        utils_lib.run_cmd(self, cmd2, expect_ret=0)
+        utils_lib.run_cmd(self, 'sudo systemctl restart chronyd.service', expect_ret=0)
+        for count in utils_lib.iterate_timeout(
+            240, "check chrony server is in use", wait=10):
+            chrony_source = utils_lib.run_cmd(self, "chronyc sources -v", expect_ret=0)
+            if re.search('\^\*', chrony_source): break
+        for count in utils_lib.iterate_timeout(
+            240, "check date be synced by chrony"):
+            date=utils_lib.run_cmd(self, 'date')
+            if not re.search("00:0", date): break
+        date_now=int(utils_lib.run_cmd(self, 'date +%s'))
+        date_now_hw=int(utils_lib.run_cmd(self, "sudo hwclock --verbose | grep 'System Time' \
+            | cut -d : -f 2 | cut -d ' ' -f 2 | cut -d . -f 1"))
+        delta = date_now_hw - date_now
+        self.assertLess(delta, 2, "delta between hwclock and date shoud less than 2 seconds")
+
+    def _test_vm_time_sync_host(self, action):
+        #check chronyd service, if enabled, disable it.
+        utils_lib.run_cmd(self, 'sudo systemctl stop chronyd.service')
+        chrony_check = utils_lib.run_cmd(self, 'sudo systemctl list-unit-files | grep chronyd.service')
+        if re.search('enabled', chrony_check):
+            utils_lib.run_cmd(self, 'sudo chkconfig chronyd off')
+        if action == 'reboot':
+            utils_lib.run_cmd(self, 'sudo date -s 00:00:00', expect_ret=0)
+            utils_lib.run_cmd(self, 'date', expect_kw="00:00:")
+            self.vm.reboot(wait=True)
+            utils_lib.init_connection(self, timeout=180)
+        elif action == 'migration':
+            self.vm.migrate()
+        cmd = 'date +%s'
+        vm_date = utils_lib.run_cmd(self, cmd)
+        host_date = utils_lib.send_ssh_cmd(self.vm.host_ip, self.vm.host_username, \
+            self.vm.host_password, cmd)[1]
+        delta = int(host_date) - int(vm_date)
+        self.assertLess(delta, 3, "delta shoud less than 3 seconds")
+
+    def test_vm_time_sync_host_after_boot(self):
+        """
+        case_name:
+            test_vm_time_sync_host_after_boot
+        case_tags:
+            time keeping
+        case_status:
+            approved
+        title:
+            Check guest vm time synced with host after reboot
+        importance:
+            medium
+        subsystem_team:
+            sst_virtualization_cloud
+        automation_drop_down:
+            Automated
+        linked_work_items:
+            N/A
+        automation_field:
+            https://github.com/virt-s1/os-tests/blob/master/os_tests/tests/test_general_test.py
+        setup_teardown:
+            Recover chrony default configure in tear down.
+        environment:
+            N/A
+        component:
+            time keeping
+        bug_id:
+            N/A
+        is_customer_case:
+            False
+        testplan:
+            N/A
+        test_type:
+            Functional
+        test_level:
+            Component
+        maintainer:
+            minl@redhat.com
+        description: |
+            Check guest vm time synced with host after reboot.
+        key_steps: |
+            # Stop chrony service.
+            # Check if chronyd.service in systemctl list-unit-files, if yes, chkconfig off it.
+            # Change VM's date.
+            # Check time between guest vm and host after reboot VM.
+        expected_result: |
+            - guest time same as the host time.
+        debug_want: |
+            - N/A
+        """
+        if not self.vm:
+            self.skipTest("Skip as no VM inited")
+        for attrname in ['host_ip']:
+            if not hasattr(self.vm, attrname):
+                self.skipTest("no {} for {} vm".format(attrname, self.vm.provider))
+        self._test_vm_time_sync_host('reboot')
+
+    def test_vm_time_sync_host_after_migration(self):
+        """
+        case_name:
+            test_vm_time_sync_host_after_migration
+        case_tags:
+            time keeping
+        case_status:
+            approved
+        title:
+            Check guest vm time synced with host after migration.
+        importance:
+            medium
+        subsystem_team:
+            sst_virtualization_cloud
+        automation_drop_down:
+            Automated
+        linked_work_items:
+            N/A
+        automation_field:
+            https://github.com/virt-s1/os-tests/blob/master/os_tests/tests/test_general_test.py
+        setup_teardown:
+            Recover chrony default configure in tear down.
+        environment:
+            N/A
+        component:
+            time keeping
+        bug_id:
+            N/A
+        is_customer_case:
+            False
+        testplan:
+            N/A
+        test_type:
+            Functional
+        test_level:
+            Component
+        maintainer:
+            minl@redhat.com
+        description: |
+            Check guest vm time synced with host
+        key_steps: |
+            # Check chrony source, if there has available server, stop chrony service.
+            # Stop chrony service.
+            # Check if chronyd.service in systemctl list-unit-files, if yes, chkconfig off it.
+        expected_result: |
+            - guest time same as the host time.
+        debug_want: |
+            - N/A
+        """
+        if not self.vm:
+            self.skipTest("Skip as no VM inited")
+        for attrname in ['host_ip']:
+            if not hasattr(self.vm, attrname):
+                self.skipTest("no {} for {} vm".format(attrname, self.vm.provider))
+        self._test_vm_time_sync_host('migration')
+
     def tearDown(self):
-        pass
+        if "test_cpu_hotplug_no_workload" in self.id():
+            cmd = "cat /sys/devices/system/cpu/cpu1/online"
+            out = utils_lib.run_cmd(self, cmd)
+            if '0' in out:
+                cmd = "sudo bash -c 'echo 1 > /sys/devices/system/cpu/cpu1/online'"
+                utils_lib.run_cmd(self, cmd, expect_ret=0, msg="enable cpu1")
+        if "test_able_to_sync_chrony_server" in self.id():
+            cmd1 = 'sudo cp /etc/chrony.conf.backup /etc/chrony.conf'
+            cmd2 = 'sudo systemctl restart chronyd.service'
+            for cmd in [cmd1, cmd2]:
+                utils_lib.run_cmd(self, cmd, expect_ret=0)
+        if "test_vm_time_sync_host" in self.id():
+            utils_lib.run_cmd(self, 'sudo chkconfig chronyd on')
+            utils_lib.run_cmd(self, 'sudo systemctl start chronyd.service')
 
 if __name__ == '__main__':
     unittest.main()
