@@ -11,7 +11,7 @@ class TestLifeCycle(unittest.TestCase):
     '''
     def setUp(self):
         utils_lib.init_case(self)
-        if self.params['remote_node'] is None:
+        if self.params.get('remote_node') is None:
             self.skipTest("Only support to run in server-client mode!")
         if utils_lib.is_ahv(self):
             self.ssh_timeout = 1200
@@ -61,15 +61,17 @@ class TestLifeCycle(unittest.TestCase):
         polarion_id:
         bz: 1703366
         '''
-        if self.vm and self.vm.provider == 'nutanix' and self.vm.prism.if_secure_boot:
+        self.default_boot_kernel  = utils_lib.run_cmd(self, "sudo grubby --default-kernel", expect_ret=0)
+        self.default_cmdline = utils_lib.run_cmd(self, 'cat /proc/cmdline')
+        if self.vm and self.vm.provider == 'nutanix' and self.vm.is_secure_boot:
             self.skipTest('''Red Hat Insights error "sed: can't read /sys/kernel/debug/sched_features: Operation not permitted" When using secure boot''')
-        self.default_boot_index  = utils_lib.run_cmd(self, 'sudo grubby --default-index', expect_ret=0)
         self.log.info("Check kernel-debug can boot up!")
         mini_mem = utils_lib.get_memsize(self)
         if int(mini_mem) < 2:
             self.skipTest('minimal 2G memory required for debug kernel')
         if utils_lib.is_arch(self, 'aarch64') and int(mini_mem) < 4:
             self.skipTest('minimal 4G memory required in aarch64')
+
         need_reboot = False
         kernel_ver = utils_lib.run_cmd(self, 'uname -r', expect_ret=0)
         if 'debug' in kernel_ver:
@@ -127,7 +129,7 @@ class TestLifeCycle(unittest.TestCase):
             self.log.info("Wait for bootup finish......")
             time.sleep(1)
         utils_lib.run_cmd(self, "dmesg", expect_not_kw="Call trace,Call Trace")
-        if int(mini_mem) < 17:
+        if int(mini_mem) <= 32:
             cmd = 'sudo bash -c "echo scan > /sys/kernel/debug/kmemleak"'
             utils_lib.run_cmd(self, cmd, expect_ret=0, timeout=1800)
 
@@ -632,15 +634,15 @@ class TestLifeCycle(unittest.TestCase):
         case_tags:
             Lifecycle,Lifecycle_tier2
         case_status:
-            Approved
+            approved
         title:
             test system reboot with two simultaneous reboot operation
         importance:
-            Low
+            low
         subsystem_team:
             sst_virtualization_cloud
         automation_drop_down:
-            Automated
+            automated
         linked_work_items:
             TBD
         automation_field:
@@ -658,9 +660,9 @@ class TestLifeCycle(unittest.TestCase):
         testplan:
             N/A
         test_type:
-            Functional
+            functional
         test_level:
-            Component
+            component
         maintainer:
             xiliang@redhat.com
         description: |
@@ -805,6 +807,9 @@ class TestLifeCycle(unittest.TestCase):
         debug_want:
             N/A
         """
+        cpu_num = int(utils_lib.run_cmd(self, 'cat /proc/cpuinfo | grep processor | wc -l'))
+        if cpu_num < 2:
+            self.skipTest("Skip test case since need at least 2 cpus for test")
         for cpus in [1,2]:
             boot_param_required = 'nr_cpus='+str(cpus)
             cat_proc_cmdline = self._update_kernel_args(boot_param_required)
@@ -845,15 +850,6 @@ class TestLifeCycle(unittest.TestCase):
             self.skipTest("Skip test case since cpu number is not greater than 2")
         if cpu_num > 2:
             cpus = 2
-        kernel_ver = utils_lib.run_cmd(self, 'uname -r', expect_ret=0)
-        cmd = 'sudo grubby --update-kernel=/boot/vmlinuz-%s --args="nr_cpus=%s"' %(kernel_ver.strip('\n'),cpus)
-        utils_lib.run_cmd(self,
-                            cmd,
-                            expect_ret=0,
-                            msg='set boot args nr_cpus=%s '%cpus)
-
-        self.vm.reboot()
-        utils_lib.init_connection(self, timeout=self.ssh_timeout)
 
         # Check /proc/cmdline for the nr_cpus args
         boot_param_required = 'nr_cpus=%s'%cpus
@@ -922,7 +918,7 @@ class TestLifeCycle(unittest.TestCase):
         try:
             is_success = self.vm.send_nmi()
         except UnSupportedAction as err:
-            self.skipTest("current {} not support nmi operation".format(self.vm.provider))
+            self.skipTest("provider:{} {}".format(self.vm.provider, err))
         if not is_success:
             self.fail("Cannot trigger panic via nmi!")
         time.sleep(10)
@@ -977,7 +973,7 @@ class TestLifeCycle(unittest.TestCase):
         try:
             is_success = self.vm.send_nmi()
         except UnSupportedAction as err:
-            self.skipTest("current {} not support nmi operation".format(self.vm.provider))
+            self.skipTest("provider:{} {}".format(self.vm.provider, err))
         if not is_success:
             self.fail("Cannot trigger panic via nmi!")
         time.sleep(10)
@@ -1073,6 +1069,12 @@ class TestLifeCycle(unittest.TestCase):
                     """
                 utils_lib.run_cmd(self, cmd, timeout=240)
 
+                utils_lib.run_cmd(self, 'sudo dracut -f', msg='Regenerate initramfs')
+                utils_lib.run_cmd(self, 'sudo reboot', msg='reboot system to make updating kernel take effect')
+                time.sleep(10)
+                utils_lib.init_connection(self, timeout=self.ssh_timeout)
+                utils_lib.run_cmd(self, 'cat /proc/cmdline', msg='Check /proc/cmdline')
+
         cmd = "sleep 3600 > /dev/null 2>&1 &"
         utils_lib.run_cmd(self, cmd)
         vm_hibernate_success = False
@@ -1088,14 +1090,18 @@ class TestLifeCycle(unittest.TestCase):
             cmd = "sudo systemctl hibernate"
             utils_lib.run_cmd(self, cmd, msg="Try to hibernate inside system!")
             time.sleep(20)
+            for count in utils_lib.iterate_timeout(
+                180, "Timed out waiting for getting server stopped."):
+                if self.vm.is_stopped(): break
 
         self.vm.start()
         time.sleep(30)
         self.params['remote_node'] = self.vm.floating_ip
-        utils_lib.init_connection(self, timeout=self.ssh_timeout)
-        utils_lib.run_cmd(self, 'dmesg', expect_kw="Restarting tasks", expect_not_kw="Call", msg="check the system is resumed")
+        utils_lib.init_connection(self, timeout=1800)
+        utils_lib.run_cmd(self, 'dmesg')
         cmd = 'pgrep -a sleep'
         utils_lib.run_cmd(self, cmd, expect_ret=0, msg='check sleep process still exists')
+        utils_lib.run_cmd(self, 'dmesg', expect_kw="Restarting tasks", expect_not_kw='Call trace,Call Trace', msg="check the system is resumed")
 
     def tearDown(self):
         reboot_require = False
@@ -1103,20 +1109,39 @@ class TestLifeCycle(unittest.TestCase):
         "nr_cpus=1","nr_cpus=2", "intel_iommu=on", "fips=1"]
         cmdline = utils_lib.run_cmd(self, 'cat /proc/cmdline')
         if cmdline:
-            current_boot_index  = utils_lib.run_cmd(self, 'sudo grubby --default-index', expect_ret=0)
-            if self.default_boot_index is not None and current_boot_index != self.default_boot_index:
-                cmd = "sudo grubby --set-default-index=%s" % self.default_boot_index
-                utils_lib.run_cmd(self, cmd, expect_ret=0, msg="restore default boot index to {}".format(self.default_boot_index))
-                reboot_require = True
             for arg in addon_args:
                 if arg in cmdline:
                     cmd = 'sudo grubby --update-kernel=ALL  --remove-args={}'.format(arg)
                     utils_lib.run_cmd(self, cmd, msg='Remove {}'.format(arg))
                     reboot_require = True
+        
+        if "boot_debugkernel" in self.id():
+            current_boot_kernel = utils_lib.run_cmd(self, "sudo grubby --default-kernel", expect_ret=0)
+            current_cmdline = utils_lib.run_cmd(self, 'cat /proc/cmdline')
+            if current_boot_kernel != self.default_boot_kernel:
+                cmd = "sudo grubby --set-default %s" % self.default_boot_kernel
+                utils_lib.run_cmd(self, cmd, expect_ret=0,
+                                  msg="Restore default boot kernel to {}".format(self.default_boot_kernel))
+                reboot_require = True
+            if "kmemleak=on" not in self.default_cmdline and "kmemleak=on" in current_cmdline:
+                cmd = 'sudo grubby --update-kernel=ALL --remove-args="kmemleak=on"'
+                utils_lib.run_cmd(self, cmd, msg='Remove "kmemleak=on" from /proc/cmdline')
+                reboot_require = True
+
+        if "test_hibernate_resume" in self.id() and self.vm and self.vm.provider != 'aws':
+            proc_cmdline = utils_lib.run_cmd(self, 'cat /proc/cmdline', msg='Cat /proc/cmdline')
+            resume_offset_arg = re.search('(resume_offset=.+)\s', proc_cmdline)
+            resume_uuid_arg = re.search('(resume=UUID=.+)$', proc_cmdline)
+            for arg in [resume_offset_arg.groups()[0], resume_uuid_arg.groups()[0]]:
+                cmd = 'sudo grubby --update-kernel=ALL  --remove-args={}'.format(arg)
+                utils_lib.run_cmd(self, cmd, msg='Remove {}'.format(arg))
+                reboot_require = True
+
         if reboot_require:
             utils_lib.run_cmd(self, 'sudo reboot', msg='reboot system under test to restore setting')
             time.sleep(10)
             utils_lib.init_connection(self, timeout=self.ssh_timeout)
+            utils_lib.run_cmd(self, 'cat /proc/cmdline', msg='Check /proc/cmdline')
 
 if __name__ == '__main__':
     unittest.main()
