@@ -2,6 +2,7 @@ import re
 import unittest
 import time
 import random
+import os
 
 from os_tests.libs import utils_lib
 from os_tests.libs.utils_lib import run_cmd
@@ -31,7 +32,8 @@ class TestNetworkTest(unittest.TestCase):
                 if net.startswith(('eth','en')):
                     self.active_nic  = net
                     break
-        self.log.info("Pick up nic {}".format(self.active_nic ))
+        self.active_nic = self.active_nic.strip()
+        self.log.info("Pick up nic {}".format(self.active_nic))
         cmd = "ip addr show {}".format(self.active_nic )
         output = utils_lib.run_cmd(self, cmd, expect_ret=0, msg='try to get {} ipv4 address'.format(self.active_nic ))
         self.ipv4 = re.findall('[\d.]{7,16}', output)[0]
@@ -176,6 +178,83 @@ class TestNetworkTest(unittest.TestCase):
         utils_lib.check_log(self, "error,warn,fail,trace", log_cmd='dmesg -T', \
             skip_words='ftrace,Adding Red Hat flag eBPF/rawtrace', cursor=self.dmesg_cursor)
 
+    def test_ethtool_L(self):
+        """
+        case_tags:
+            network
+        title:
+            Test change the numbers of channels of the specified network device.
+        importance:
+            low
+        subsystem_team:
+            sst_virtualization_cloud
+        automation_drop_down:
+            automated
+        linked_work_items:
+            N/A
+        automation_field:
+            https://github.com/virt-s1/os-tests/blob/master/os_tests/tests/test_network_test.py
+        setup_teardown:
+            N/A
+        environment:
+            N/A
+        case_name:
+            test_ethtool_L
+        component:
+            kernel
+        bug_id:
+            N/A
+        is_customer_case:
+            N/A
+        testplan:
+            N/A
+        maintainer:
+            xiliang@redhat.com
+        description: |
+            Use ethtool to query and change the numbers of channels of the specified network device.
+        key_steps: |
+            # ethtool -l $nic
+            # ethtool -L $nic rx|tx|other|combined N
+        expected_result: |
+            Can change the numbers of channels of the specified network device successfully.
+        debug_want: |
+            # ethtool -i $nic
+            # ethtool -l $nic
+            # dmesg
+        """
+        utils_lib.is_cmd_exist(self, 'ethtool')
+        cmd = "ethtool -i {}".format(self.active_nic)
+        utils_lib.run_cmd(self, cmd, expect_ret=0)
+        query_cmd = "ethtool -l {}".format(self.active_nic)
+        utils_lib.run_cmd(self, query_cmd, cancel_not_kw='not supported')
+
+        for i in ['rx', 'tx', 'other','combined']:
+            cmd = "ethtool -l {}|grep -i {}|head -1".format(self.active_nic,i)
+            out = utils_lib.run_cmd(self, cmd)
+            max_val = re.findall('[\d]+',out)
+            cmd = "ethtool -l {}|grep -i {}|tail -1".format(self.active_nic,i)
+            out = utils_lib.run_cmd(self, cmd)
+            init_val = re.findall('[\d]+',out)
+            if not max_val or not init_val:
+                self.log.info("Not specified in max or current:{}".format(i))
+                continue
+            max_val = int(max_val[0])
+            init_val = int(init_val[0])
+            if max_val <= 1:
+                self.log.info("do not change any thing when value is less or equal 1:{}".format(i))
+                continue
+            set_val = random.randint(1,max_val)
+            cmd = "sudo ethtool -L {} {} {}".format(self.active_nic, i, set_val)
+            out = utils_lib.run_cmd(self, cmd)
+            if out and "supported" in out:
+                self.log.info("continue next option as cannot change this option")
+                continue
+            utils_lib.run_cmd(self, query_cmd, expect_kw=str(set_val))
+            cmd = "sudo ethtool -L {} {} {}".format(self.active_nic, i, init_val)
+            utils_lib.run_cmd(self, cmd,msg='restore it to default setting')
+            utils_lib.run_cmd(self, query_cmd, expect_kw=str(init_val))
+        utils_lib.run_cmd(self, 'dmesg')
+
     def test_ethtool_P(self):
         """
         case_tags:
@@ -183,7 +262,7 @@ class TestNetworkTest(unittest.TestCase):
         title:
             Test ethtool query nic's mac address
         importance:
-            Low
+            low
         subsystem_team:
             sst_virtualization_cloud
         automation_drop_down:
@@ -234,30 +313,53 @@ class TestNetworkTest(unittest.TestCase):
                              cmd,
                              expect_kw=mac,
                              msg='compare with ip showed mac')
+
     def test_ethtool_S_xdp(self):
-        '''
+        """
         case_name:
             test_ethtool_S_xdp
-        case_file:
-            https://github.com/liangxiao1/os-tests/blob/master/os_tests/tests/test_network_test.py
-        case_priority:
-            2
+        case_tags:
+            network
+        case_status:
+            approved
+        title:
+            [os-tests]TestNetworkTest.test_ethtool_S_xdp
+        importance:
+            high
+        subsystem_team:
+            sst_virtualization_cloud
+        automation_drop_down:
+            automated
+        linked_work_items:
+            n/a
+        automation_field:
+            https://github.com/virt-s1/os-tests/blob/master/os_tests/tests/test_network_test.py/#L317
+        setup_teardown:
+            n/a
+        environment:
+            n/a
         component:
             kernel
-        bugzilla_id:
-            1908542, 2000400
-        polarion_id:
+        bug_id:
+            2038158, 2038056, 2038079, 1908542, 2000400
+        is_customer_case:
+            True
+        testplan:
             n/a
+        test_type:
+            functional
+        test_level:
+            Component
         maintainer:
             xiliang@redhat.com
-        description:
+        description: |
             Use ethtool to query the specified network device xdp statistics.
-        key_steps:
+        key_steps: |
             1. # ethtool -S $nic |grep xdp
             2. # xdp-loader status (xdp-tools is required and it is only support x86_64 for now)
             3. # xdp-loader unload -a
             4. # xdp-filter load --mode skb $nic
-        expected_result:
+        expected_result: |
             1. xdp data found
             eg. # ethtool -S eth0 |grep xdp
                   queue_0_rx_xdp_aborted: 0
@@ -267,13 +369,13 @@ class TestNetworkTest(unittest.TestCase):
                   queue_0_rx_xdp_invalid: 0
                   queue_0_rx_xdp_redirect: 0
             2. xdp-loader works on x86_64 platform
-        debug_want:
+        debug_want: |
             1. # uname -r
             2. # ethtool -i $nic
             3. # rpm -q xdp-tools (if fail at xdp-tools)
             4. # xdp-loader $cmd -vv
+        """
 
-        '''
         product_id = utils_lib.get_os_release_info(self, field='VERSION_ID')
         cmd = "sudo ethtool -i {}".format(self.active_nic )
         output = utils_lib.run_cmd(self, cmd, expect_ret=0)
@@ -339,7 +441,7 @@ class TestNetworkTest(unittest.TestCase):
             mtu_max = 65535
         elif 'vmxnet3' in output:
             self.log.info('vmxnet3 found!')
-            if self.params['remote_node'] is not None:
+            if self.params.get('remote_node') is not None:
                 self.skipTest("Skip mtu test while running remotely with vmxnet3")
             self.log.info("vmxnet3 min mtu is 60, because of bz1503193, skip test lower value than 68")
             mtu_range = [68, 4500, 9190, 9192]
@@ -482,7 +584,7 @@ class TestNetworkTest(unittest.TestCase):
         utils_lib.run_cmd(self, cmd, msg='create /tmp/test/hello')
         registries = ['docker.io/library/httpd:2.4','docker.mirrors.ustc.edu.cn/library/httpd:2.4']
         for registry in registries:
-            cmd = 'podman run -dit --name httpd_site -p 8188:80 -v "/tmp/test":/usr/local/apache2/htdocs/:Z {}'.format(registry)
+            cmd = 'sudo podman run -dit --name httpd_site -p 8188:80 -v "/tmp/test":/usr/local/apache2/htdocs/:z {}'.format(registry)
             ret = utils_lib.run_cmd(self, cmd, timeout=600, msg='start container httpd_site', ret_status=True)
             if ret == 0:
                 break
@@ -550,10 +652,9 @@ COMMIT
         '''
         self.log.info("Enter _test_add_remove_multi_nics procedure")
         #record origin nic's mac and name
-        origin_nic_name = utils_lib.run_cmd(self, "ls /sys/class/net/ | grep -v lo").strip()
-        origin_nic_mac = utils_lib.run_cmd(self, "cat /sys/class/net/%s/address" % origin_nic_name).strip()
+        origin_nic_mac = utils_lib.run_cmd(self, "cat /sys/class/net/%s/address" % self.active_nic).strip()
         #stop vm to add nic if vm is secure boot and record the set ip list
-        if self.vm.provider == 'nutanix' and self.vm.prism.if_secure_boot:
+        if self.vm.provider == 'nutanix' and self.vm.if_secure_boot:
             self.vm.stop(wait=True)
         try:
             used_ip_list = self.vm.list_networks_address(self.vm.private_network_uuid)
@@ -574,7 +675,7 @@ COMMIT
             self.vm.attach_nic(network_uuid, ip_address, driver)
         time.sleep(10)
         set_ip_list.sort()
-        if self.vm.provider == 'nutanix' and self.vm.prism.if_secure_boot:
+        if self.vm.provider == 'nutanix' and self.vm.if_secure_boot:
             #start vm
             self.vm.start(wait=True)
             time.sleep(30)
@@ -586,7 +687,7 @@ COMMIT
         nic_name_list = (utils_lib.run_cmd(self, "ls /sys/class/net | grep e")).split()
         for nic_name in nic_name_list:
             nic_driver = utils_lib.run_cmd(self, "ethtool -i %s | grep driver | awk '{print $2}'" % nic_name).strip()
-            if nic_name == origin_nic_name:
+            if nic_name == self.active_nic:
                 self.assertEqual(nic_driver, 'virtio_net', msg="Default nic dirver is not virtio_net, real: %s" % nic_driver)
             else:
                 if driver == 'virtio':
@@ -605,13 +706,13 @@ COMMIT
             vm_ip_list.sort()
             self.assertEqual(vm_ip_list, set_ip_list, msg="IP configure is not right, \
                 Expect: %s, real: %s" % (str(set_ip_list), str(vm_ip_list)))
-        if self.vm.provider == 'nutanix' and self.vm.prism.if_secure_boot:
+        if self.vm.provider == 'nutanix' and self.vm.if_secure_boot:
             self.vm.stop(wait=True)
         #delete nic by mac
         for mac in nic_mac_list:
             if mac != origin_nic_mac:
                 self.vm.detach_nic(mac)
-        if self.vm.provider == 'nutanix' and self.vm.prism.if_secure_boot:
+        if self.vm.provider == 'nutanix' and self.vm.if_secure_boot:
             self.vm.start(wait=True)
             time.sleep(30)
             utils_lib.init_connection(self, timeout=180)
@@ -684,7 +785,7 @@ COMMIT
         if not self.vm:
             self.skipTest('vm not init')
         utils_lib.check_attribute(self.vm, 'private_network_uuid,private_network_subnet',test_instance=self, cancel_case=True)
-        if self.vm.provider == 'nutanix' and self.vm.prism.machine_type == 'q35':
+        if self.vm.provider == 'nutanix' and self.vm.machine_type == 'q35':
             multi_num = 1
         else:
             multi_num = 2
@@ -725,8 +826,7 @@ COMMIT
     def _test_unload_load_nic_driver(self, driver):
         self.log.info("Enter _test_unload_load_nic_driver procedure")
         #record the first nic
-        origin_nic_name = utils_lib.run_cmd(self, "ls /sys/class/net/ | grep -v lo").strip()
-        origin_nic_mac = utils_lib.run_cmd(self, "cat /sys/class/net/%s/address" % origin_nic_name).strip()
+        origin_nic_mac = utils_lib.run_cmd(self, "cat /sys/class/net/%s/address" % self.active_nic).strip()
         #atach the second nic
         if self.vm.provider == 'nutanix' and self.vm.prism.if_secure_boot:
             self.vm.stop(wait=True)
@@ -737,7 +837,7 @@ COMMIT
             time.sleep(60)
         self.vm.refresh_data()
         utils_lib.init_connection(self, timeout=180)
-        new_nic_mac = utils_lib.run_cmd(self, "cat /sys/class/net/%s/address" % origin_nic_name).strip()
+        new_nic_mac = utils_lib.run_cmd(self, "cat /sys/class/net/%s/address" % self.active_nic).strip()
         self.assertNotEqual(origin_nic_mac, new_nic_mac, msg="Second nic name changed after detaching the first nic, Expect not: %s, real: %s" % (origin_nic_mac, new_nic_mac))
         if driver == 'virtio':
             driver_check = 'virtio_net'
@@ -746,8 +846,8 @@ COMMIT
         for i in range (2):
             self.log.info("Unload and load nic driver for %s time(s)." % i)
             utils_lib.run_cmd(self, "sudo modprobe -r %s && sudo modprobe %s" % (driver_check, driver_check), expect_ret=0)
-        new_nic_name = utils_lib.run_cmd(self, "ls /sys/class/net/ | grep -v lo").strip()
-        self.assertEqual(origin_nic_name, new_nic_name, msg="Second nic name changed after unload/load nic driver three times, Expect: %s, real: %s" % (origin_nic_name, new_nic_name))
+        new_nic_name = utils_lib.run_cmd(self, "ls /sys/class/net/ | grep -Ev 'lo|podman|veth'").strip()
+        self.assertEqual(self.active_nic, new_nic_name, msg="Second nic name changed after unload/load nic driver three times, Expect: %s, real: %s" % (self.active_nic, new_nic_name))
 
     def _create_vm1(self):
         create_vm_name = self.vm.vm_name+'_nework_script_create'
@@ -804,11 +904,22 @@ COMMIT
         if len(self.params['remote_nodes']) < 2:
             self.skipTest("2 nodes required, current IP bucket:{}".format(self.params['remote_nodes']))
         self.log.info("Current IP bucket:{}".format(self.params['remote_nodes']))
-        arp_result = utils_lib.run_cmd(self, 'arping -I %s %s -c 10' % (self.active_nic, self.params['remote_nodes'][-1]))
-        arp_response = int(re.search('Received\s+(\d+)\s+response', arp_result, re.I).groups()[0])
-        self.assertAlmostEqual(first=int(arp_response), second=10, delta=1, msg='check arping response error, expect:%s, real:%s')
-        utils_lib.run_cmd(self, 'sudo ping -f %s -c 2' % (self.params['remote_nodes'][-1]), expect_ret=0)
-        utils_lib.run_cmd(self, 'sudo ping -f %s -c 600' % (self.params['remote_nodes'][-1]), expect_kw='0% packet loss')
+        #Found case will fail after vm boot time became more long, change to loop check
+        for count in utils_lib.iterate_timeout(
+            60, "Check VM1 has started and can arping", wait=10):
+            res = utils_lib.run_cmd(self, \
+                'arping -I %s %s -c 2' % (self.active_nic, self.params['remote_nodes'][-1]))
+            if not re.search('Received\s+0\s+response', res): break
+        arp_result = utils_lib.run_cmd(self, \
+            'arping -I %s %s -c 10' % (self.active_nic, self.params['remote_nodes'][-1]))
+        arp_response = int(re.search('Received\s+(\d+)\s+response', \
+            arp_result, re.I).groups()[0])
+        self.assertEqual(first=int(arp_response), second=10, \
+            msg='check arping response error, expect:%s, real:%s')
+        utils_lib.run_cmd(self, 'sudo ping -f %s -c 2' % (self.params['remote_nodes'][-1]), \
+            expect_ret=0)
+        utils_lib.run_cmd(self, 'sudo ping -f %s -c 600' % (self.params['remote_nodes'][-1]), \
+            expect_kw='0% packet loss')
 
     def test_iperf(self):
         """
@@ -1036,6 +1147,20 @@ COMMIT
                 'rx-gro-list': 'rx-gro-list',
                 'rx-udp-gro-forwarding': 'rx-udp-gro-forwarding'
             }
+        elif 'driver: gve' in output:
+            option_dict = {
+                'rx': 'rx-checksumming',
+                'tx': 'tx-checksumming',
+                'sg': 'scatter-gather',
+                'tso': 'tcp-segmentation-offload',
+                'gso': 'generic-segmentation-offload',
+                'gro': 'generic-receive-offload',
+                'tx-nocache-copy': 'tx-nocache-copy',
+                'rx-gro-list': 'rx-gro-list',
+                'rx-udp-gro-forwarding': 'rx-udp-gro-forwarding',
+                'rxhash': 'receive-hashing',
+                'highdma': 'highdma'
+            }
         else:
             option_dict = {
                 'rx': 'rx-checksumming',
@@ -1066,6 +1191,7 @@ COMMIT
             cmd = 'sudo ethtool -k eth0'
             utils_lib.run_cmd(self, cmd, expect_kw="%s: on" % option_dict[option])
 
+    @unittest.skipUnless(os.getenv('INFRA_PROVIDER') == 'aws', 'aws dedicated feature')
     def test_check_efa_device_driver(self):
         """
         case_tag:
@@ -1104,6 +1230,7 @@ COMMIT
             run_cmd(self, cmd, expect_ret=0, msg='check if EFA device exist and efa module is loaded')
             self.log.info('EFA device is found and efa driver is loaded on the instance ' + instance_type)
             
+    @unittest.skipUnless(os.getenv('INFRA_PROVIDER') == 'aws', 'aws dedicated feature')
     def test_install_libfabric_check_efa_provider(self):
         """
         case_tag:
@@ -1141,7 +1268,8 @@ COMMIT
             if utils_lib.is_pkg_installed(self,'libfabric'):
                 cmd = 'fi_info -p efa'
                 utils_lib.run_cmd(self, cmd, expect_ret=0, expect_kw="provider: efa", msg='Check the Libfabric EFA interfaces')
-            
+
+    @unittest.skipUnless(os.getenv('INFRA_PROVIDER') == 'aws', 'aws dedicated feature')        
     def test_load_unload_efa_driver(self):
         """
         case_tag:
@@ -1192,6 +1320,7 @@ COMMIT
             if ret == 0:
                 self.log.info('efa driver is loaded successfully')
 
+    @unittest.skipUnless(os.getenv('INFRA_PROVIDER') == 'aws', 'aws dedicated feature')
     def test_attach_detach_efa_device(self):
         """
         case_tag:
@@ -1227,8 +1356,8 @@ COMMIT
         debug_want:
             N/A
     	"""
-        if not self.vm or self.vm.provider != "aws":
-            self.skipTest("Skip test case since instance is not vm or aws")
+        if not self.vm:
+            self.skipTest("Skip test case since instance is not vm")
         else:
             instance_type = self.vm.instance_type
             if not self.vm.efa_support:
@@ -1302,8 +1431,7 @@ COMMIT
         if 'mtu 9000' not in host_mtu_check[1]:
             self.skipTest("MTU of the vm host is not 9000")
         utils_lib.is_pkg_installed(self,"net-tools")
-        nic_name = utils_lib.run_cmd(self, "ls /sys/class/net/ | grep -v lo").strip()
-        utils_lib.run_cmd(self, 'sudo ifconfig %s mtu 9000' % nic_name, expect_ret=0)
+        utils_lib.run_cmd(self, 'sudo ifconfig %s mtu 9000' % self.active_nic, expect_ret=0)
         if len(self.vms) == 1:
             self._create_vm1()
         vm1_host_uuid = self.vm.prism.get_vm_by_uuid(self.vms[1]['uuid'])['host_uuid']
@@ -1312,9 +1440,7 @@ COMMIT
             self.vm.migrate(host_uuid=vm1_host_uuid, wait=True)
         utils_lib.init_connection(self, timeout=180, rmt_node=self.vm.vm1_ip)
         utils_lib.run_cmd(self, 'sudo yum install -y net-tools', rmt_node=self.vm.vm1_ip, timeout=180)
-        cmd="ls /sys/class/net/ | grep -v lo"
-        vm1_nic_name = utils_lib.run_cmd(self, cmd, expect_ret=0, rmt_node=self.vm.vm1_ip).strip()
-        utils_lib.run_cmd(self, 'sudo ifconfig %s mtu 9000' % vm1_nic_name, expect_ret=0, rmt_node=self.vm.vm1_ip)
+        utils_lib.run_cmd(self, 'sudo ifconfig %s mtu 9000' % self.active_nic, expect_ret=0, rmt_node=self.vm.vm1_ip)
         #create key
         key_file_check = utils_lib.run_cmd(self, "ls ~/.ssh/id_rsa.pub")
         if 'No such file or directory' in key_file_check:
@@ -1327,6 +1453,7 @@ COMMIT
         file_size = int(utils_lib.run_cmd(self, "ls -l ~/5G.img | awk '{print $5}'", expect_ret=0).strip())/(1024*1024*1024)
         self.assertAlmostEqual(first=5, second=file_size, delta=0.1, msg="Value of copied file is not right, Expect: 5, real: %s" % (file_size))
 
+    @unittest.skipUnless(os.getenv('INFRA_PROVIDER') == 'aws', 'aws dedicated feature')
     def test_mpi_app_via_efa_provider(self):
         """
         case_tag:
@@ -1359,8 +1486,8 @@ COMMIT
         debug_want:
             efa,libfabric
         """
-        if not self.vm or self.vm.provider != "aws":
-            self.skipTest("Skip test case since instance is not vm or aws")
+        if not self.vm:
+            self.skipTest("Skip test case since instance is not vm")
 
         instance_type = self.vm.instance_type
         if not self.vm.efa_support:
@@ -1401,9 +1528,11 @@ COMMIT
         debug_want:
             N/A
         """
+        if utils_lib.is_aws(self) or utils_lib.is_gcp(self):
+            self.skipTest("Unable to download pkg from internal site now")
         check_file = self.utils_dir + '/nw_pktgen.sh'
         check_file_tmp = '/tmp/nw_pktgen.sh'
-        if self.params['remote_node'] is not None:
+        if self.params.get('remote_node') is not None:
             cmd = 'ls -l {}'.format(check_file_tmp)
             ret = utils_lib.run_cmd(self, cmd, ret_status=True, msg='check if {} exists'.format(check_file))
             if ret != 0:
@@ -1459,16 +1588,16 @@ COMMIT
         except UnSupportedAction:
             self.skipTest('nic create func is not supported in {}'.format(self.vm.provider))
 
-        netdev_index = 1
+        cmd = "ip -o link show|wc -l"
+        nic_num_1 = utils_lib.run_cmd(self,cmd)
         self.vm.attach_nic(self.nic,device_index=1, wait=True)
         for i in range(1, 4):
             time.sleep(5)
             self.log.info('Check network in guest, loop {}'.format(i))
-            cmd = "lspci"
-            output1 = utils_lib.run_cmd(self, cmd)
-            cmd = "ip addr show"
-            output1 = utils_lib.run_cmd(self, cmd)
-            if 'eth%s' % netdev_index not in output1:
+            utils_lib.run_cmd(self, 'lspci')
+            utils_lib.run_cmd(self, 'ip addr show')
+            nic_num_2 = utils_lib.run_cmd(self,cmd)
+            if nic_num_2 == nic_num_1:
                 self.log.info("Added nic not found")
         timeout = 120
         interval = 5
@@ -1484,11 +1613,9 @@ COMMIT
            time.sleep(interval)
         time.sleep(5)
         cmd = "ip addr show"
-        utils_lib.run_cmd(self, cmd)
+        utils_lib.run_cmd(self,cmd)
         self.nic.delete()
-        self.assertIn('eth%d' % netdev_index,
-                      output1,
-                      msg='eth{} not found after attached nic'.format(netdev_index))
+        self.assertGreater(nic_num_2,nic_num_1,msg='New added network device not found after attached nic')
         cmd = 'dmesg'
         utils_lib.run_cmd(self, cmd, expect_not_kw='Call Trace')
 
@@ -1496,10 +1623,10 @@ COMMIT
         '''
         description:
             [RHEL8.4] Test 2nd ip hotplug on primary nic, nm-cloud-setup can assign/remove it automatically
-        bugzilla_id: 
-            1623084,1642461
-        customer_case_id: 
-            BZ1623084,BZ1642461
+        bug_id:
+            bugzilla_1623084,bugzilla_1642461,bugzilla_2179718
+        is_customer_case:
+            True
         maintainer: 
             xiliang
         case_priority: 
@@ -1507,7 +1634,7 @@ COMMIT
         case_component: 
             network
         key_steps: |
-            1. Launch an instance on AWS EC2.
+            1. Launch an instance on AWS EC2/Alicloud/Azure/GCP.
             2. Check if package NetworkManager-cloud-setup is installed via command "$ sudo rpm -q NetworkManager-cloud-setup", if not, use yum install to install it.
             3. Check the service status via command "$ sudo systemctl status nm-cloud-setup.timer".
             4. Assign the second IP to the NIC of instance.
@@ -1517,6 +1644,13 @@ COMMIT
         pass_criteria: |
             After the second IP is assigned to the NIC of instance in step 4, there will be 2 IP address shows in step5.
             After the second IP is removed from the NIC of instance 6, there will be only 1 IP address shows in the step7.
+        debug_want: |
+            Attach trace log from nm-cloud-setup when case fail.
+            - Run "systemctl edit nm-cloud-setup" uncomment "Environment=NM_CLOUD_SETUP_LOG=TRACE"
+            - "systemctl daemon-reload"
+            - systemctl restart nm-cloud-setup.service
+            - journalctl -u nm-cloud-setup.service
+            - AWS only(ensure infra assign it): curl 169.254.169.254/latest/meta-data/network/interfaces/macs/$MAC/local-ipv4s
         '''
         utils_lib.is_pkg_installed(self, pkg_name='NetworkManager-cloud-setup', is_install=False, cancel_case=True)
         for attrname in ['assign_new_ip','remove_added_ip']:
